/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDAS
*       Filename : Base64.c
*		Shared functions for encoding data in Base64 format. 
*
*       Updated :
*               11/07/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>

#include "../Commoninclude.h"


static char *b64chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

/******************************************************************************
*       char * Base64Enc(unsigned char * data, int len)
*		Encodes data into a Base64 string.
*
*	Args :
*		unsigned char * data	Data string to encode
*		int len			length of string.
*
*       Updated :
******************************************************************************/
char * Base64Enc(unsigned char * data, int len)
{                         
        char * OutData = 0;
        unsigned char EncodedBlock[5];
        unsigned i = 0;   

        OutData = drealloc(OutData, (len * 2));

        while (len >= 3)  
        {
                OutData[i] = b64chars[(data[0] >> 2) & 0x3f];
                OutData[i+1] = b64chars[((data[0] << 4) & 0x30) | ((data[1] >> 4) & 0xf)];
                OutData[i+2] = b64chars[((data[1] << 2) & 0x3c) | ((data[2] >> 6) & 0x3)];
                OutData[i+3] = b64chars[data[2] & 0x3f];
                OutData[i+4] = 0;

                i = i + 4;

                data += 3;
                len -= 3; 
        }                 

        if (len > 0)
        {                 
                EncodedBlock[0] = b64chars[(data[0] >> 2) & 0x3f];
                EncodedBlock[1] = b64chars[(data[0] << 4) & 0x30];
                if (len > 1)
                {         
                        EncodedBlock[1] = b64chars[((data[0] << 4) & 0x30) | ((data[1] >> 4) & 0xf)];
                        EncodedBlock[2] = b64chars[(data[1] << 2) & 0x3c];
                } else {  
                        EncodedBlock[2] = '=';
                }         

                EncodedBlock[3] = '=';
                EncodedBlock[4] = 0;

                OutData = dstrcat(OutData, EncodedBlock);
        }                 

        return(OutData);  
}                       

/******************************************************************************
*       struct BinData * Base64Dec(unsigned char * data)
*		Decodes string from Base64 encoding. Returns struct with
*		data len and pointer to data.
*
*	Args :
*		unsigned char * data	Base64 encoded string.
*
*       Updated :
******************************************************************************/
struct BinData * Base64Dec(unsigned char * data)
{
	int cur = 0;
	int val = 0;
	unsigned i = 0;
	unsigned BuffLen = 0;
	unsigned datalen = 0;
	unsigned char * OutBuff = 0;
	
	struct BinData * OutData = 0;
	
	if(data)
	{
		OutBuff = drealloc(OutBuff, dstrlen(data));
		OutData = (struct BinData *) malloc(sizeof(struct BinData));

		datalen = dstrlen(data);

		for(i=0;i<datalen;i++)
		{
			val = 0;
			
			if(data[i] >= 'A' && data[i] <= 'Z')
			{
				val = data[i] - 'A';
			} else if(data[i] >= 'a' && data[i] <= 'z') {
				val = data[i] - 'a' + 26;
			} else if(data[i] >= '0' && data[i] <= '9') {
				val = data[i] - '0' + 52;
			} else if(data[i] == '+') {
				val = 62;
			} else if(data[i] == '/') {
				val = 63;
			} else if(data[i] == '=') {
				break;
			} else {
				continue;
			}
			
			switch(cur++)
			{
				case 0:
					OutBuff[BuffLen] = (val << 2) & 0xfc;	
					break;	
				case 1:
					OutBuff[BuffLen] |= (val >> 4) & 0x03;
					BuffLen++;
					OutBuff[BuffLen] = (val << 4) & 0xf0;
					break;
				case 2:
					OutBuff[BuffLen] |= (val >> 2) & 0x0f;
					BuffLen++;
					OutBuff[BuffLen] = (val << 6) & 0xc0;
					break;
				case 3:
					OutBuff[BuffLen] |= val & 0x3f;
					BuffLen++;
					cur = 0;
					break;
			}
		}
		OutBuff[BuffLen] = 0;
		
		OutData->Len = BuffLen;
		OutData->Data = OutBuff;
	}
	
	return(OutData);
}	
