/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDAS
*       Filename : Crypt.c
* 		Shared twofish Encryption/Decryption functions. Input/Output 
*		strings are Base64'ed for easily listening. Encypt only works 
*		on null termed strings. Decrypt returns a Null termed string.
*
*       Updated :
*               11/10/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>
#include <time.h>

#include "aes.h"

#include "../Commoninclude.h"

extern char * KEYFILE;

/******************************************************************************
*       char * Encrypt(char * text)
*		Will encrypt a null term string. Outputs a null term Base64 
*		string.
*
*	Args :
*		char * text	The null term string to encrypt. 
*
*       Updated :
******************************************************************************/
char * Encrypt(char * text)
{
	char * Key = 0;
	char * textBlock = 0;
	unsigned textLen = 0;
	char * CryptBlock = 0;
	unsigned CryptLen = 0;
	char * OutBlock = 0;

	keyInstance ki;
	cipherInstance ci;

	if(text)
	{	
		textLen = ((dstrlen(text) / BLOCK_SIZE) + 1) * BLOCK_SIZE;
		
		textBlock = drealloc(textBlock, textLen);
		textBlock = dstrcpy(textBlock, text);
		
		Key = ReadKey();
		if(Key)
		{
			bzero(ki.key32, sizeof(ki.key32));
			bzero(ki.keyMaterial, sizeof(ki.keyMaterial));
			strcpy(ki.keyMaterial, Key);
			
			if(makeKey(&ki, DIR_ENCRYPT, 256, ki.keyMaterial))
			{
			        bzero(ci.iv32, sizeof(ci.iv32));
				ci.mode = MODE_ECB;
				
				CryptBlock = drealloc(CryptBlock, textLen);
				CryptLen = blockEncrypt(&ci, &ki, textBlock, textLen, CryptBlock);
				
				if(CryptLen)
				{
					OutBlock = Base64Enc(CryptBlock, CryptLen);
				}
			}
		} else {
			OutBlock = dstrcpy(OutBlock, text);
		}
		
		Key = dfree(Key);
		textBlock = dfree(textBlock);
		CryptBlock = dfree(CryptBlock);
	}
	
	return(OutBlock);	
}

/******************************************************************************
*       char * Decrypt(char * text)
*		Decrypts Based64'ed encrypted text into null termed text 
*		string.
*
*	Args :
*		char * text	Based64'ed encrypted text.
*
*       Updated :
******************************************************************************/
char * Decrypt(char * text)
{
	char * Key = 0;
	struct BinData * CryptData = 0;
	char * OutBlock = 0;
	
	keyInstance ki;
	cipherInstance ci;

	if(text)
	{
		CryptData = Base64Dec(text);
		if(CryptData)
		{
			if(CryptData->Len)
			{
				Key = ReadKey();
	
				if(Key)
				{
					bzero(ki.key32, sizeof(ki.key32));
					bzero(ki.keyMaterial, sizeof(ki.keyMaterial));
					strcpy(ki.keyMaterial, Key);
					
					if(makeKey(&ki, DIR_DECRYPT, 256, ki.keyMaterial))
					{
						bzero(ci.iv32, sizeof(ci.iv32));
						ci.mode = MODE_ECB;
						
						OutBlock = drealloc(OutBlock, CryptData->Len);
						
						blockDecrypt(&ci, &ki, CryptData->Data, CryptData->Len, OutBlock);
					}
				} else {
					OutBlock = dstrcpy(OutBlock, text);
				}
			
				Key = dfree(Key);
			} else {
				OutBlock = dstrcpy(OutBlock, text);
			}
			
			CryptData->Data = dfree(CryptData->Data);
			free(CryptData);
		}
	}
	return(OutBlock);			
}

/******************************************************************************
*       char * ReadKey(void)
*		Reads key file and returns contents as string. Assumes char 
*		KEYFILE global is specified.
*
*
*       Updated :
******************************************************************************/
char * ReadKey(void)
{
	FILE *fp;
	char * Key = 0;
	
	if(KEYFILE)
	{
		fp = fopen(KEYFILE, "r");
		if(fp)
		{
			Key = dfiletostring(fp);
			fclose(fp);
		}
	}
	
	return(Key);
}

/******************************************************************************
*       char * MakeKey(void)
*		Builds 256 bit encryption/decryption key.
*
*
*       Updated :
******************************************************************************/
char * MakeKey()
{
	int i;
	long ltime;
	int utime;
	
	char * OutKey = 0;
	char * tOutKey = 0;
	
	ltime = time(NULL);
	utime = (unsigned int) ltime / 2;
	
	srand(utime);
		
	for(i=0;i<(256/8);i++)
	{
		tOutKey = dsprintf(tOutKey, "%.2x", (rand() % 255));
		OutKey = dstrcat(OutKey, tOutKey);
		tOutKey = dfree(tOutKey);
	}
	
	return(OutKey);
}
