/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDAS
*       Filename : Forker.c
* 		Functions for handling the fork()ing of child processes.
*
*       Updated :
*               11/10/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/wait.h>
#include <time.h>
#include <signal.h>

#include "../Commoninclude.h"

extern struct XMLLeaf * ConfigXMLTree;
extern int LOG_FORK;

int CHILDTTL = 180;

/******************************************************************************
*       int FreeChild(unsigned CHILD[255], unsigned BIRTH[255])
*		Returns slot number of free child slot, or 0 if none available.
*
*	Args :
*		unsigned CHILD[255]	Array of child pid's (Or 0 if empty).
*		unsigned BIRTH[255]	Array of Child Birth dates.
*
*       Updated :
******************************************************************************/
int FreeChild(unsigned CHILD[255], unsigned BIRTH[255])
{
	int i;
	
	CleanChildren(CHILD, BIRTH);
	
	for(i=0;i<Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.MAXFORKS"));i++)
	{
		if(CHILD[i] == 0)
		{
			return(i);
		} 
	}
	return(-1);
}

/******************************************************************************
*       void AddChild(pid_t child, unsigned CHILD[255], unsigned BIRTH[255])
*		Adds chil's pid to the 1st emtpy slot in chils slot array.
*		Sets the coresponding BIRTH array element to current time().
*
*	Args :
*		pid_t child		pid of child being added.
*		 unsigned CHILD[255]	Array of child pid's (Or 0 if empty).
*		unsigned BIRTH[255]	Array of Child Birth dates.
*
*       Updated :
******************************************************************************/
void AddChild(pid_t child, unsigned CHILD[255], unsigned BIRTH[255])
{
	int i;
	
	CleanChildren(CHILD, BIRTH);
	
	for(i=0;i<Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.MAXFORKS"));i++)
	{
		if(CHILD[i] == 0)
		{
			CHILD[i] = (unsigned) child;
			BIRTH[i] = time(NULL);
			break;
		}
	}
}
	
/******************************************************************************
*       void CleanChildren(unsigned CHILD[255], unsigned BIRTH[255])
*		Iterate through child slot array, freeing up any closed 
*		children or children that have expired (Current time > CHILDTTL 
*		+ BIRTH).
*
*	Args :
*		unsigned CHILD[255]	Array of child pid's (Or 0 if empty).
*		unsigned BIRTH[255]	Array of Child Birth dates.
*
*       Updated :
******************************************************************************/
void CleanChildren(unsigned CHILD[255], unsigned BIRTH[255])
{	
	int i;
	pid_t PID;
	int status;
	
	for(i=0;i<Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.MAXFORKS"));i++)
	{
 		if(CHILD[i])
 		{
        		PID = waitpid(CHILD[i], &status, WNOHANG);
        	
        		if((BIRTH[i] > 0 && CHILD[i] > 0) &&  BIRTH[i] < (time(NULL) - CHILDTTL))
        		{
        			MyDebug(__LINE__, __FILE__, LOG_FORK, "[CleanChildren]-[Child TTL expired : %u]", CHILD[i]);
 				kill(CHILD[i], 9);       	
        		}
        		
   			if(PID == -1)
   			{
 				if(!WIFEXITED(status))
 				{
 					MyDebug(__LINE__, __FILE__, LOG_FORK, "[CleanChildren]-[Child:%u exited with status:%d]", CHILD[i], WEXITSTATUS(status));
 				}
 				
   				CHILD[i] = 0;
   				BIRTH[i] = 0;
   			}
   		}
   	}
}

/******************************************************************************
*       int NoChild(unsigned CHILD[255], unsigned BIRTH[255])
*		Returns 1 if Children process still in child slot array, or
*		0 if all slots are empty (0'ed).
*
*	Args :
*		unsigned CHILD[255]	Array of child pid's (Or 0 if empty).
*		unsigned BIRTH[255]	Array of Child Birth dates.
*
*       Updated :
******************************************************************************/
int NoChild(unsigned CHILD[255], unsigned BIRTH[255])
{
	int found = 0;
	int i;

	CleanChildren(CHILD, BIRTH);

	found = 0;
	for(i=0;i<Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.MAXFORKS"));i++)
	{
		if(CHILD[i] != 0)
		{
			found = 1;
			break;
		}
	}
	return(found);
}

