/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDAS
*       Filename : XMLparser.c
* 		Function for processing XML.
*
*       Updated :
*               11/10/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>

#include "../Commoninclude.h"

extern int LOG_XML;

/******************************************************************************
*       struct XMLLeaf * StringToXMLTree(char * instr)
*		Converts an XML string to a XML Linked list.
*
*	Args :
*		char * instr	XML String to convert.
*
*       Updated :
******************************************************************************/
struct XMLLeaf * StringToXMLTree(char * instr)
{
	char * TmpTag = 0;
	char * TmpValue = 0;
	char * tCut = 0;
	
	char * Localinstr = 0;
	char * so_Localinstr = 0;  // Used to restore correct pointer location after strsep steps on Localinstr.
		
	struct XMLLeaf * CurrentLeaf = 0;
	struct XMLLeaf * ParentLeaf = 0;
	struct XMLLeaf * RootLeaf = 0;
	struct XMLLeaf * NewLeaf = 0;
	long TagCount = 0;
	
	Localinstr = dstrcpy(Localinstr, instr);
	so_Localinstr = Localinstr;
	
	if(Localinstr)
	{
		so_Localinstr = Localinstr;
		
		if(strstr(Localinstr, "<") && strstr(Localinstr, ">"))
		{
			strsep(&Localinstr, "<");
			do
			{
				TmpTag = strsep(&Localinstr, ">");
				if(TmpTag)
				{
					if(!strncmp(TmpTag, "!--", 3))		// Comment Start
					{
						TmpTag = dstradvance(TmpTag, dstrlen(TmpTag) - 2);
						if(strcmp(TmpTag, "--"))	// Did we end on the the ending --> comment, if not
						{				// we parse again. 	
							Localinstr = strstr(Localinstr, "-->");
						}
						strsep(&Localinstr, "<");
					} else if(TmpTag[0] == '?') { 		// XML Formating Ignore
						strsep(&Localinstr, "<");
					} else if(TmpTag[0] == '/') { 		// End Tag
						if(ParentLeaf)
						{
							ParentLeaf = ParentLeaf->ParentLeaf;
						}
						strsep(&Localinstr, "<");
					} else { 				// Start of Tag value
						if(strchr(TmpTag, ' ')) 	// Theres an Arg specified, cut it off.
						{				// We should eventualy put in the Aurg handling function here. 
							tCut = (char *) strchr(TmpTag, ' ');
							*tCut = 0;
						}
					
						TmpValue = dstrcpy(TmpValue, strsep(&Localinstr, "<"));
						TmpValue = dstrclean(TmpValue);
					
						NewLeaf = AddXMLLeaf(ParentLeaf, CurrentLeaf, TmpTag, TmpValue, TagCount);
						TmpValue = dfree(TmpValue);
						
						ParentLeaf = NewLeaf;
						CurrentLeaf = NewLeaf;
							
						if(!RootLeaf)
						{
							RootLeaf = CurrentLeaf;
						}
						TagCount++;
					}
				}
			}while(TmpTag);
		}
	}
	
	so_Localinstr = dfree(so_Localinstr);
	Localinstr = NULL;
	
	return(RootLeaf);	
}

/******************************************************************************
*       char * XMLTreeToString(struct XMLLeaf * RootLeaf)
*		Converts an XML linked list to its string value.
*
*	Args :
*		struct XMLLeaf * RootLeaf	The XML Linked list to process.
*
*       Updated :
******************************************************************************/
char * XMLTreeToString(struct XMLLeaf * RootLeaf)
{
	struct XMLLeaf * LocalLeaf = 0;
	struct XMLLeaf * LocalRootLeaf = 0;
	
	char * tLocalString = 0;
	char * LocalString = 0;
	char * tChildString = 0;
	char * ChildString = 0;
	
	LocalRootLeaf = RootLeaf;
	
	if(LocalRootLeaf)
	{
		if(FindXMLLeafByParent(LocalRootLeaf, LocalRootLeaf))
		{
			LocalLeaf = FindXMLLeafByParent(LocalRootLeaf, LocalRootLeaf);
			
			while(LocalLeaf)
			{
				tChildString = XMLTreeToString(LocalLeaf);
				if(tChildString)
				{
					ChildString = dstrcat(ChildString, tChildString);
					tChildString = dfree(tChildString);
				}
				LocalLeaf = FindXMLLeafByParent(LocalLeaf, LocalRootLeaf);
			}
		}
		
		tLocalString = dsprintf(tLocalString, "<%s>%s", LocalRootLeaf->Tag, LocalRootLeaf->Value);
		LocalString = dstrcat(LocalString, tLocalString);
		tLocalString = dfree(tLocalString);
		
		if(ChildString)
		{
			LocalString = dstrcat(LocalString,  ChildString);
			ChildString = dfree(ChildString);
		}

		tLocalString = dsprintf(tLocalString, "</%s>", LocalRootLeaf->Tag);
		LocalString = dstrcat(LocalString, tLocalString);
		tLocalString = dfree(tLocalString);
	}

	return(LocalString);
}

/******************************************************************************
*       struct XMLLeaf * AddXMLLeaf(struct XMLLeaf * ParentLeaf, struct XMLLeaf *PrevLeaf, char *Tag, char *Value, long TagCount)
*		Adds element to XML linked list.
*
*	Args :
*		struct XMLLeaf * ParentLeaf	Pointer to Parent XML linked 
*						list leaf.
*		struct XMLLeaf *PrevLeaf	Pointer to Prev. XML Linked
*						list leaf.
*		char * Tag			Tag name.
*		char * Value			Value of tag.
*		long TagCount			Current Tag count.
*
*       Updated :
******************************************************************************/
struct XMLLeaf * AddXMLLeaf(struct XMLLeaf * ParentLeaf, struct XMLLeaf *PrevLeaf, char *Tag, char *Value, long TagCount)
{
	struct XMLLeaf * NewLeaf = 0;

	if(Tag)
	{
		NewLeaf = (struct XMLLeaf *) malloc(sizeof(struct XMLLeaf));
		bzero(NewLeaf, sizeof(struct XMLLeaf));
	
		NewLeaf->ID = TagCount;
		if(ParentLeaf) 
		{
			NewLeaf->Fullname = (char *) malloc(dstrlen(ParentLeaf->Fullname) + dstrlen(Tag) + 2);
			sprintf(NewLeaf->Fullname, "%s.%s", ParentLeaf->Fullname, Tag);
		} else {
			NewLeaf->Fullname = dstrcpy(NewLeaf->Fullname, Tag);
		}
		
		NewLeaf->Tag = dstrcpy(NewLeaf->Tag, Tag);
		
		if(Value)
		{
			NewLeaf->Value = dstrcpy(NewLeaf->Value, Value);
		}
		
		NewLeaf->ParentLeaf = ParentLeaf;
	
		if(ParentLeaf)
		{
			NewLeaf->ParentID = ParentLeaf->ID;
		}
		
		NewLeaf->PrevLeaf = PrevLeaf;
		NewLeaf->NextLeaf = NULL;
		
		if(PrevLeaf)
		{
			(PrevLeaf)->NextLeaf = NewLeaf;
		}
		
		return(NewLeaf);
	}
	
	return(NULL);
	
}

/******************************************************************************
*       void DelXMLLeaf(struct XMLLeaf * Leaf)
*		Deletes an element in the XML linked list.
*
*	Args :
*		struct XMLLeaf * Leaf		Pointer to element to delete.
*
*       Updated :
******************************************************************************/
void DelXMLLeaf(struct XMLLeaf * Leaf)
{
	if(Leaf)
	{
		dfree(Leaf->Tag);
		dfree(Leaf->Value);
		dfree(Leaf->Fullname);
		
		free(Leaf);
	}
}

/******************************************************************************
*       struct XMLLeaf * DelXMLLeafTree(struct XMLLeaf *Leaf)
*		Deletes an entire XML tree.
*
*	Args :
*		struct XMLLeaf * Leaf		Pointer to root element to 
*						delete from.
*
*       Updated :
******************************************************************************/
struct XMLLeaf * DelXMLLeafTree(struct XMLLeaf *Leaf)
{
	struct XMLLeaf * LocalLeaf = 0;
	
	if(Leaf)
	{
		LocalLeaf = Leaf;
		while(LocalLeaf->NextLeaf)
		{
			dfree(LocalLeaf->Tag);
			dfree(LocalLeaf->Value);
			dfree(LocalLeaf->Fullname);
			
			LocalLeaf = LocalLeaf->NextLeaf;
			
			free(LocalLeaf->PrevLeaf);
		}
		
		dfree(LocalLeaf->Tag);
		dfree(LocalLeaf->Value);
		dfree(LocalLeaf->Fullname);
			
		free(LocalLeaf);
	}
	return(NULL);
}

/******************************************************************************
*       struct XMLLeaf * FindXMLLeafByTag(struct XMLLeaf * RootLeaf, char * Tag)
*		Finds the first element that has a matching tag. Returns the 
*		pointer to found element.
*
*	Args :
*		struct XMLLeaf * RootLeaf	Pointer to starting element.
*		char * Tag			Tag to look for.
*
*       Updated :
******************************************************************************/		
struct XMLLeaf * FindXMLLeafByTag(struct XMLLeaf * RootLeaf, char * Tag)
{
	struct XMLLeaf * LocalLeaf = 0;
	
	if(Tag)
	{
		LocalLeaf = RootLeaf;
		while(LocalLeaf)
		{
			if(LocalLeaf)
			{
				if(!strcasecmp(LocalLeaf->Fullname, Tag))
				{
					return(LocalLeaf);
				}
				LocalLeaf = LocalLeaf->NextLeaf;
			}
		}
	}
	return(NULL);	
}

/******************************************************************************
*       struct XMLLeaf * FindXMLLeafByID(struct XMLLeaf * RootLeaf, long ID)
*		Finds the first element that has a matching ID. Returns the 
*		pointer to found element.
*
*       Args :
*               struct XMLLeaf * RootLeaf       Pointer to starting element.
*		long ID				ID to search for.
*
*       Updated :
******************************************************************************/
struct XMLLeaf * FindXMLLeafByID(struct XMLLeaf * RootLeaf, long ID)
{
	struct XMLLeaf * LocalLeaf = 0;
	
	LocalLeaf = RootLeaf;
	while(LocalLeaf)
	{
		if(LocalLeaf)
		{
			if(LocalLeaf->ID == ID)
			{
				return(LocalLeaf);
			}
			LocalLeaf = LocalLeaf->NextLeaf;
		}
	}
	return(NULL);	
}

/******************************************************************************
*       long ReturnXMLLeafParentID(struct XMLLeaf * RootLeaf, char * Tag)
*		Finds element with matching tag and returns its parent ID.
*
*	Args :
*		struct XMLLeaf * RootLeaf       Pointer to starting element.
*		char * Tag			Tag to search for.
*
*       Updated :
******************************************************************************/
long ReturnXMLLeafParentID(struct XMLLeaf * RootLeaf, char * Tag)
{
	struct XMLLeaf * LocalLeaf = 0;
	
	if(Tag)
	{
		LocalLeaf = RootLeaf;
		while(LocalLeaf)
		{
			if(LocalLeaf)
			{
				if(!strcasecmp(LocalLeaf->Fullname, Tag))
				{
					return(LocalLeaf->ParentID);
				}
				LocalLeaf = LocalLeaf->NextLeaf;
			}
		}
	}
	return(0);
}

/******************************************************************************
*       long ReturnXMLLeafID(struct XMLLeaf * RootLeaf, char * Tag)
*		Finds element with matching tag and returns its ID.
*
*	Args :
*		struct XMLLeaf * RootLeaf       Pointer to starting element.
*		char * Tag			Tag to search for.
*
*       Updated :
******************************************************************************/
long ReturnXMLLeafID(struct XMLLeaf * RootLeaf, char * Tag)
{
	struct XMLLeaf * LocalLeaf = 0;
	
	if(Tag)
	{
		LocalLeaf = RootLeaf;
		while(LocalLeaf)
		{
			if(LocalLeaf)
			{
				if(!strcasecmp(LocalLeaf->Fullname, Tag))
				{
					return(LocalLeaf->ID);
				}
				LocalLeaf = LocalLeaf->NextLeaf;
			}
		}
	}
	return(0);
}

/******************************************************************************
*       struct XMLLeaf * FindXMLLeafByParent(struct XMLLeaf * RootLeaf, struct XMLLeaf * ParentLeaf)
*		Finds element with matching ParentLeaf pointer and returns 
*		its element pointer.
*
*	Args :
*		struct XMLLeaf * RootLeaf       Pointer to starting element.
*		struct XMLLeaf * ParentLeaf	Parent leaf pointer to match.
*
*       Updated :
******************************************************************************/
struct XMLLeaf * FindXMLLeafByParent(struct XMLLeaf * RootLeaf, struct XMLLeaf * ParentLeaf)
{
	struct XMLLeaf * LocalLeaf = 0;
	
	if(RootLeaf)
	{
		LocalLeaf = RootLeaf->NextLeaf;
		while(LocalLeaf)
		{
			if(LocalLeaf->ParentLeaf == ParentLeaf)
			{
				return(LocalLeaf);
			}
			LocalLeaf = LocalLeaf->NextLeaf;
		}
			
	}
	return(NULL);
}

/******************************************************************************
*	struct XMLLeaf * FindXMLLeafByParentID(struct XMLLeaf * RootLeaf, long ParentID)
*		Finds element with matching Parent ID and returns its element
*		pointer.
*
*	Args :
*		struct XMLLeaf * RootLeaf       Pointer to starting element.
*		log ParentID			ParentID to match.
*	
*
*       Updated :
******************************************************************************/
struct XMLLeaf * FindXMLLeafByParentID(struct XMLLeaf * RootLeaf, long ParentID)
{
	struct XMLLeaf * LocalLeaf = 0;
	
	if(RootLeaf)
	{
		LocalLeaf = RootLeaf->NextLeaf;
		while(LocalLeaf)
		{
			if(LocalLeaf->ParentID == ParentID)
			{
				return(LocalLeaf);
			}
			LocalLeaf = LocalLeaf->NextLeaf;
		}
			
	}
	return(NULL);
}

/******************************************************************************
*       char * ReturnXMLLeafValue(struct XMLLeaf * RootLeaf, char * Tag)
*		Finds element with matching tag and returns its value.
*
*	Args :
*		struct XMLLeaf * RootLeaf       Pointer to starting element.
*		char * Tag			Tag to match.
*
*       Updated :
******************************************************************************/	
char * ReturnXMLLeafValue(struct XMLLeaf * RootLeaf, char * Tag)
{
	struct XMLLeaf * LocalLeaf = 0;

	LocalLeaf = FindXMLLeafByTag(RootLeaf, Tag);
	if(LocalLeaf)
	{
		return(LocalLeaf->Value);
	} else {
		return(NULL);
	}
}

/******************************************************************************
*       struct XMLLeaf * BuildSubXMLTree(struct XMLLeaf * RootLeaf, long ID, int IsRoot)
*		Builds a 'Sub tree' this contains only elements that are 
*		offsprings of the Parent ID.
*
*	Args :
*		struct XMLLeaf * RootLeaf       Pointer to starting element.
*		long ID				Parent ID to start at.
*		int IsRoot			Is this the Root of Sub Tree?
*						This should always be set to 0.
*
*
*       Updated :
******************************************************************************/
struct XMLLeaf * BuildSubXMLTree(struct XMLLeaf * RootLeaf, long ID, int IsRoot)
{
	struct XMLLeaf * NewLeaf = 0;
	struct XMLLeaf * ChildLeaf = 0;
	struct XMLLeaf * CurrentLeaf = 0;
	
	if(RootLeaf)
	{
		if(IsRoot)
		{
			ChildLeaf = FindXMLLeafByID(RootLeaf, ID);
			
			if(ChildLeaf)
			{
				NewLeaf = (struct XMLLeaf *) malloc(sizeof(struct XMLLeaf));
				bzero(NewLeaf, sizeof(struct XMLLeaf));
					
				NewLeaf->ID = ChildLeaf->ID;
				NewLeaf->Tag = ChildLeaf->Tag;
				NewLeaf->Fullname = ChildLeaf->Fullname;
				NewLeaf->Value = ChildLeaf->Value;
				NewLeaf->ParentLeaf = ChildLeaf->ParentLeaf;
				NewLeaf->ParentID = ChildLeaf->ParentID;
				NewLeaf	->NextLeaf = NULL;
			
				//MyDebug(__LINE__, __FILE__, LOG_XML, "[BuildSubXMLTree]-[Root]-[ID:%ld]-[ParentID:%ld]-[FullName:%s]", 
				//		NewLeaf->ID, NewLeaf->ParentID,
				//		NewLeaf->Fullname);

				CurrentLeaf = NewLeaf;
			}
		}
			
		ChildLeaf = FindXMLLeafByParentID(RootLeaf, ID);
		if(ChildLeaf)
		{
			while(ChildLeaf)
			{
				NewLeaf = (struct XMLLeaf *) malloc(sizeof(struct XMLLeaf));
				bzero(NewLeaf, sizeof(struct XMLLeaf));
				
				NewLeaf->ID = ChildLeaf->ID;
				NewLeaf->Tag = ChildLeaf->Tag;
				NewLeaf->Fullname = ChildLeaf->Fullname;
				NewLeaf->Value = ChildLeaf->Value;
				NewLeaf->ParentLeaf = ChildLeaf->ParentLeaf;
				NewLeaf->ParentID = ChildLeaf->ParentID;
				NewLeaf->NextLeaf = NULL;
				
				//MyDebug(__LINE__, __FILE__, LOG_XML, "[BuildSubXMLTree]-[Child]-[ID:%ld]-[ParentID:%ld]-[FullName:%s]", 
				//		NewLeaf->ID, NewLeaf->ParentID,
				//		NewLeaf->Fullname);

				if(CurrentLeaf)
				{
					NewLeaf->PrevLeaf = CurrentLeaf;
					CurrentLeaf->NextLeaf = NewLeaf;
				}
				CurrentLeaf = NewLeaf;
				
				if(FindXMLLeafByParentID(RootLeaf, ChildLeaf->ID))
				{
					NewLeaf = BuildSubXMLTree(RootLeaf, ChildLeaf->ID, 0);
					
					CurrentLeaf->NextLeaf = NewLeaf;
					NewLeaf->PrevLeaf = CurrentLeaf;
					
					while(CurrentLeaf->NextLeaf)
					{
						CurrentLeaf = CurrentLeaf->NextLeaf;
					}
				}
				ChildLeaf = FindXMLLeafByParentID(ChildLeaf, ID);
			}
		} else {
			if(!IsRoot)
			{
				NewLeaf = (struct XMLLeaf *) malloc(sizeof(struct XMLLeaf));
				bzero(NewLeaf, sizeof(struct XMLLeaf));
				NewLeaf->ID = RootLeaf->ID;
				NewLeaf->Tag = RootLeaf->Tag;
				NewLeaf->Fullname = RootLeaf->Fullname;
				NewLeaf->Value = RootLeaf->Value;
				NewLeaf->ParentLeaf = RootLeaf->ParentLeaf;
				NewLeaf->ParentID = RootLeaf->ParentID;
				NewLeaf->NextLeaf = NULL;
				
				//MyDebug(__LINE__, __FILE__, LOG_XML, "[BuildSubXMLTree]-[Childless]-[ID:%ld]-[ParentID:%ld]-[FullName:%s]", 
				//			NewLeaf->ID, NewLeaf->ParentID,
				//			NewLeaf->Fullname);
				CurrentLeaf = NewLeaf;
			}
		}
	}
	
	if(CurrentLeaf)
	{
		while(CurrentLeaf->PrevLeaf)
		{
			CurrentLeaf = CurrentLeaf->PrevLeaf;
		}
	}
	return(CurrentLeaf);
}

/******************************************************************************
*       struct XMLLeaf * DelSubXMLTree(struct XMLLeaf * RootLeaf)
*		Deletes/Frees a sub tree.
*
*	Args :
*		struct XMLLeaf * RootLeaf	Pointer to Sub tree to free.
*
*
*       Updated :
******************************************************************************/		
struct XMLLeaf * DelSubXMLTree(struct XMLLeaf * RootLeaf)
{
	while(RootLeaf->NextLeaf)
	{
		RootLeaf = RootLeaf->NextLeaf;
		free(RootLeaf->PrevLeaf);
	}
	free(RootLeaf);
	
	return(NULL);
}

/******************************************************************************
*	char * CleanXMLValueString(char * Value)
*		Cleans the Un-XML characters from a string.
*
*	Args :
*		char * Value	The string to clean.
*
*	[TCC] This function SHOULD NOT be used on large string values
*	with high occurances of needle.
*
*       Updated :
******************************************************************************/
char * CleanXMLValueString(char * Value)
{
	Value = dsubstrreplace(Value, "<", "&lt;");
	Value = dsubstrreplace(Value, ">", "&gt;");
	Value = dsubstrreplace(Value, "\255", "&255;"); 
	return(Value);
}

/******************************************************************************
*       char * UnCleanXMLValueString(char * Value)
*		Reverts 'Cleaned' characters to there normal character.
*
*	Args :
*		char * Value	The string to dirty.
*
*       Updated :
******************************************************************************/
char * UnCleanXMLValueString(char * Value)
{
	Value = dsubstrreplace(Value, "&lt;", "<");
	Value = dsubstrreplace(Value, "&gt;", ">");
	Value = dsubstrreplace(Value, "&255;", "\255");
	
	return(Value);
}

/******************************************************************************
*       void DebugXMLTree(char * LocString, struct XMLLeaf * XMLTree, int Syslog)
*		Prints/Logs the contents of an XML linked list in a readable
*		format.
*
*	Args :
*		char * LocString		String used within loging 
*						message as a 'Location ID'.
*		struct XMLLeaf * XMLTree	The XML Linked list to display.
*		int Syslog			Toggle this being sent to syslog.
*
*       Updated :
******************************************************************************/
void DebugXMLTree(char * LocString, struct XMLLeaf * XMLTree, int Syslog)
{
	struct XMLLeaf * LocalLeaf;
	
	MyDebug(__LINE__, __FILE__, LOG_XML, "[DebugXMLTree]-[%s]-[Enter]", LocString);
	
	LocalLeaf = XMLTree;
	while(LocalLeaf)
	{
		if(LocalLeaf)
		{
			if(Syslog)
			{
				MyDebug(__LINE__, __FILE__, LOG_XML, "[DebugXMLTree]-[%s]-[%ld]-[%ld]-[%s]", LocString, LocalLeaf->ID, LocalLeaf->ParentID, LocalLeaf->Fullname);
			} else {
				printf("[%s] [%ld] %s = \"%s\"\n", LocString, LocalLeaf->ID, LocalLeaf->Fullname, LocalLeaf->Value);
			}
			
			LocalLeaf = LocalLeaf->NextLeaf;
		}
	}
	MyDebug(__LINE__, __FILE__, LOG_XML, "[DebugXMLTree]-[%s]-[Exit]", LocString);
}

/******************************************************************************
*       unsigned CountXMLLeafByTag(struct XMLLeaf * CheckLeaf, char * Tag)
*		Count the number of elements that match a specified tag.
*
*	Args :
*		struct XMLLeaf * CheckLeaf	Pointer of elements to start at.
*		 char * Tag			Tag to count.
*
*       Updated :
******************************************************************************/
unsigned CountXMLLeafByTag(struct XMLLeaf * CheckLeaf, char * Tag)
{
	struct XMLLeaf * LocalLeaf = 0;
	unsigned count = 0;
	
	LocalLeaf = CheckLeaf;
	
	while(LocalLeaf)
	{
		LocalLeaf = FindXMLLeafByTag(LocalLeaf, Tag);
		
		if(LocalLeaf)
		{
			count++;
			LocalLeaf = LocalLeaf->NextLeaf;
		}
	}
	
	return(count);
}
