<?
/********************************************************************
Copyright (c) 2003 Jason Sessler

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
********************************************************************/

/*************************************************************************
   file:	inc/authentication.inc.php
   date:	02/21/2003
   Author:	jsessler
   build:	10142003
   
   CHANGES:
   10/14/2003   - Checking $authkey using is_null() in CheckAuthentication()
   
   10/08/2003   - Changed format of the date string returned by the function
                  getLastLogin()

   09/09/2003   - Modified Authenticate & CheckAuthentication functions to
                  to support the ability to send back a message along with
                  the return value of the session. This was created to
				  inform users if sessions are still active with their
				  username. (JS)

   09/05/2003	- Modified to support the uid field required by MIDAS for
                  authentication, and also removed the following vars/fields
				  required by the ACTIVITY table; username, passwd. (JS)
				  
				- Converted all of the mySQL code over to the new mySQL
				  CLASS "conn_mySQL". (JS)
				  
				- Removed obsolete code from previous version(s). (JS)
				
				- Removed WriteLog() Function. Ugly, Ugly, Ugly (JS)
				
   TODO:
                * Remove any un-needed mySQL connections. Try to create a
				  global mySQL connection for the whole authentication CLASS.

				* Build a better WriteLog function.
 *************************************************************************/

CLASS phpIMAuth
{
	// Base Login Information
		var $uid;
		var $username;
		var $passwd;
        var $accessLevel;
		var $sesskey;
		var $remoteaddr;

	var $TMP;

	Function phpIMAuth()
	{
		$this->TMP		    = _TMPDIR;
		$this->remoteaddr 	= $this->getRemoteAddr();
	}

	Function Authenticate($authname,$authpass)
	{
		$this->username   = $authname;
		$this->passwd     = $authpass;

		list($userinfo,$num) = $this->getAuthKey();

		if($num) {
			// Autheticated...
				$this->Slap($userinfo);
				$stamp = $this->getStamp();

			    global $credentials;
			    $db = new conn_mySQL($credentials);
				$db->db_new_conn();
			    $db->add_table(_TABLE_ACTIVITY);
				$db->add_data("uid", $this->uid);
				$db->add_data("accessLevel", $this->accessLevel);
				$db->add_data("remoteaddr", $this->remoteaddr);
				$db->add_data("sesskey", $this->sesskey);
				$db->add_data("stamp", $stamp);
				$ret = $db->compile_query("INSERT");
				if($ret['return']) {
					$db->run_query();
				} else {
					/**********************************************************
					  Normally this will happen if there is an active session
					  owned by the user that is trying to login.
					 **********************************************************/
					 $db->clear_res();
					 $db->db_close();
					 $db = NULL;
					 return Array("return" => FALSE, "msg" => "User already Logged In");
				}

				$db->clear_res();
				$db->db_close();
				$db = FALSE;

				$this->CreateCookie();

				return Array("return" => TRUE);
		} else {
			// Authentication Error...
				return Array("return" => FALSE, "msg" => "Invalid USERNAME and/or PASSWORD");
		}
	}

    Function Logout($authkey)
    {
        if($this->ViewAuthentication($authkey)) {
            $this->KillSession($authkey);
            return TRUE;
        } else {
            return FALSE;
        }
    }

	Function CheckAuthentication($authkey)
	{
		if(!is_null($authkey)) {
			list($userinfo,$num) = $this->getAuthKeyBySession($authkey);

			$this->Slap($userinfo);

			if($num) {
				// User is Authenticated...
					$this->CreateCookie();
					return Array("return" => TRUE);
			} else {
				// Authentication Error...
				    $this->ClearCookie();
					return Array("return" => FALSE);
			}
		} else {
			return Array("return" => FALSE);
		}
	}

	Function ViewAuthentication($authkey)
	{
		if(isset($authkey)) {
			list($userinfo,$num) = $this->getAuthKeyBySession($authkey);

			$this->Slap($userinfo);

			if($num) {
				// User is Authenticated...
					return Array("return" => TRUE);
			} else {
				// Authentication Error...
					return Array("return" => FALSE);
			}
		}
	}

    Function checkAccessLevel($access)
    {
        $b_access = sprintf("%08d", decbin($access));
        $b_accessLevel = sprintf("%08d", decbin($this->accessLevel));

        for($x=0;$x<strlen($b_access);$x++) {
            if($b_access[$x])
                if($b_accessLevel[$x])
                    continue;
                else
                    return FALSE;
        }
        return TRUE;
    }

	Function CreateCookie()
	{
		global $_SERVER;
		@SetCookie("_SESSKEY",$this->sesskey, time()+60*60*24*30, "/", $_SERVER['HTTP_HOST']);

		$this->WriteSession();
	}

	Function ClearCookie()
	{
		global $_SERVER;
		@SetCookie("_SESSKEY", 0,time()-3600, "/", $_SERVER['HTTP_HOST']);

		$this->username     = "";
		$this->passwd	    = "";
		$this->accessLevel	= "";
		$this->sesskey	    = "";
	}

	Function getAuthKey()
	{
		global $driver;

		switch($driver['auth']) {
			case "mySQL":
			    global $credentials;
			    $db = new conn_mySQL($credentials);
				$db->db_new_conn();
			    $db->add_table(_TABLE_USERS);
			    $db->add_select_item(Array(_TABLE_USERS => Array("uid","accessLevel")));
			    $db->add_index("username");
			    $db->clause = sprintf(" LEFT JOIN %s on %s.uid=%s.uid where %s.username='%s' and %s.passwd=MD5('%s')",
					        		  _TABLE_ACTIVITY, _TABLE_USERS, _TABLE_ACTIVITY, _TABLE_USERS, $this->username, _TABLE_USERS, $this->passwd);
				$ret = $db->compile_query("SELECT");
				if($ret['return']) {
					$db->run_query();
					if($db->rows) {
						$db->step();
						$userinfo = $db->res;
						$num = $db->rows;
					} else {
					    $userinfo = FALSE;
					    $num = 0;
					}
				} else {
					// Need to log this error *** TODO ***
				}
				$db->clear_res();
				$db->db_close();
				$db = FALSE;
				break;
			case "LDAP":
				// Not implemented yet....
				break;
			case "file":
				// Not implemented yet....
				break;
			default:
				printf("<pre>Error: invalid authentication driver: %s</pre>\n", $driver['auth']);
		}

        if($userinfo && $num) {
    		srand((double)microtime()*1000000);
    		$authsess = md5(rand(0,9999999));

            $userinfo['sesskey'] = $authsess;
        }
        
		return Array($userinfo, $num);
	}

	Function getAuthKeyBySession($authkey)
	{
		global $credentials;
		$db = new conn_mySQL($credentials);
		$db->db_new_conn();
		$db->add_table(_TABLE_ACTIVITY);
		$db->add_select_item(Array("uid", "accessLevel", "remoteaddr"));
		$db->add_index("sesskey");
		$db->clause = sprintf("where sesskey='%s'", $authkey);
		$ret = $db->compile_query("SELECT");
		if($ret['return']) {
			$db->run_query();
			$num = $db->rows;
	        if($num) {
	            $db->step();
	            $info = array_merge($db->res, Array("sesskey" => $authkey));
				if($this->remoteaddr != $info['remoteaddr']) {
					$num = 0;
					$info = FALSE;
	                $this->ClearCookie();
				}
			}
		} else {
			// Need to log this error *** TODO ***
		}
		
		$db->clear_res();
		$db->db_close();
		$db = FALSE;

		return Array($info,$num);
	}

	Function getRemoteAddr()
	{
		if ($_SERVER['HTTP_X_FORWARDED_FOR']) {
			$ip   = $_SERVER['HTTP_X_FORWARD_FOR'];
		} else {
			$ip   = $_SERVER['REMOTE_ADDR'];
		}
		return $ip;
	}

	Function getUNAME()
	{
		global $credentials;
		$db = new conn_mySQL($credentials);
		$db->db_new_conn();
		$db->add_table(_TABLE_USERS);
		$db->add_select_item(Array("username"));
		$db->add_index("uid");
		$db->clause = sprintf("where uid='%s'", $this->uid);
		$ret = $db->compile_query("SELECT");
		if($ret['return']) {
			$db->run_query();
	        if($db->rows) {
	            $db->step();
	        	return $db->res['username'];
			} else
			    return "unknown";
		}
		$db->clear_res();
		$db->db_close();
		$db = FALSE;
	}
	
	Function getLastLogin()
	{
		global $credentials;
		$db = new conn_mySQL($credentials);
		$db->db_new_conn();
		$db->add_table(_TABLE_USERS);
		$db->add_select_item(Array("lastAccess"));
		$db->add_index("uid");
		$db->clause = sprintf("where uid='%s'", $this->uid);
		$ret = $db->compile_query("SELECT");
		if($ret['return']) {
			$db->run_query();
	        if($db->rows) {
	            $db->step();
	        	return date("D M j g:i:s A T Y", $db->res['lastAccess']);
			} else
			    return "unknown";
		}
		$db->clear_res();
		$db->db_close();
		$db = FALSE;
	}
	
	Function Slap($info)
	{
		$this->uid          = $info['uid'];
		$this->accessLevel  = $info['accessLevel'];
		$this->sesskey		= $info['sesskey'];
	}
	
	Function WriteSession()
	{
		global $credentials;
		$db = new conn_mySQL($credentials);
		$db->db_new_conn();
		$db->add_table(_TABLE_ACTIVITY);
		$db->add_data("stamp", $stamp);
		$db->clause = sprintf("where uid='%s' AND sesskey='%s'", $this->uid, $authkey);
		$ret = $db->compile_query("UPDATE");
		if($ret['return']) {
			$db->run_query();
		} else {
			// Need to log this error *** TODO ***
		}
		
		$db->clear_res();
		$db->db_close();
		$db = FALSE;
	}

    Function getStamp()
    {
        return mktime(date("G"), date("i"), date("s"), date("m"), date("d"), date("Y"));
    }

	Function KillSession($authsess)
	{
		global $credentials;
		$db = new conn_mySQL($credentials);
		$db->db_new_conn();
		$db->add_table(_TABLE_ACTIVITY);
		$db->clause = sprintf("where uid='%s' and sesskey='%s'", $this->uid, $this->sesskey);
		$ret = $db->compile_query("DELETE");
		if($ret['return']) {
			$db->run_query();
			$stamp = $this->getStamp();
			$db->clear_res();
			$db->add_table(_TABLE_USERS);
			$db->add_data("lastAccess",$stamp);
			$db->clause = sprintf("where uid='%s'", $this->uid);
			$ret = $db->compile_query("UPDATE");
		    if($ret['return']) {
		    	$db->run_query();
		    } else {
		    	// Need to log this error *** TODO ***
		    }
			$this->ClearCookie();
		} else {
			// Need to log this error *** TODO ***
		}
		
		$db->clear_res();
		$db->db_close();
		$db = FALSE;
	}
}

$session = new phpIMAuth();

?>
