<?
/********************************************************************
Copyright (c) 2003 Jason Sessler

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
********************************************************************/

/*************************************************************************
   file:	lib/menu.inc.php
   date:	09/04/2003
   Author:	jsessler
   build:	09092003

	CHANGES:
	09/09/2003      - Added Separator support for MenuLinks (JS)
	
	09/07/2003  	- Added Security & Separator support (JS)
	                - Added MENU CLASS HOW-TO (JS)
	                - Added External Menu Functions (JS)
	TODO:
	                * Need to add another level to the menu array to allow
	                  for priority listing. (JS)
					* Create ability to make Separators in the menu system. (DONE)
					* Could run into MenuLink Names being the same. This will
					  cause the javascript DHTML menu to blow up. Need to work
					  on changing the javascript id from the MenuLink's Name to
					  some other form of id.

 *************************************************************************/

/*
	MENU CLASS HOW-TO
	Internal CLASS Functions:
	    $var->add_MenuLink(array $link, var $menu)
	    $link: Array ("name"      => Name of the MenuLink (No Spaces)
	                  "exe"       => What to execute: Javascript action/link (I.e. <window>.location.href='index.htm';/window.open('index.htm','Index','');)
	                  "SEP"       => Build Separator before MenuLink
	                  "menuWidth  => Width of the actual dropdown window
	                  "width"     => Width of the MenuLink box
	                 )
		$menu: Location of MenuLink (All links start off of ROOT. For a MenuLink that is under
		                             the Go Menu: "ROOT/Go")

		$var->add_MenuItem(array $item, var $menu)
		$item: Array ("name"    => Name of the MenuItem (Spaces Allowed)
	                  "link"    => Javascript action/link (I.e. <window>.location.href='index.htm';/window.open('index.htm','Index','');)
	                  "SEP"     => Build Separator before MenuItem
	                  "SEC"     => Lock MenuItem to only users with X security level (Default: _USER)
	                 )
		$menu: Location of MenuLink (All links start off of ROOT. For a MenuLink that is under
		                             the Go Menu: "ROOT/Go")

	External Functions:
	    add_MenuLink($name, $width, $menuWidth, [$menu = "ROOT"], [$exe = FALSE])
	    add_MenuItem($name, $link, $menu, [$SEP = FALSE], [$SEC = _USER])
	    
	Menu CLASS variable:
		$_TOP_MENU
*/

CLASS menu
{
	var $menuItem;
	
	function menu()
	{
		global $color;
		// Setting up initial menu variables
		$this->menuItem = Array(
							"ROOT" => Array(
							            "itemOnColor"   => $color[9], "itemOffColor"  => $color[0],
							            "menuNext"      => "<img src='images/menu/next.gif'>",
							            "newMenuXDepl"  => -2, "newMenuYDepl"  => 2,
										"menuBgColor"   => $color[3], "menuFgColor"   => $color[8]));

		// Building Default ParentLists
		$this->add_MenuLink(Array("name" => "Go", "exe" => FALSE, "menuWidth" => 120, "width" => 55));
		$this->add_MenuLink(Array("name" => "Search", "exe" => FALSE, "menuWidth" => 120, "width" => 55));
		$this->add_MenuLink(Array("name" => "Config", "exe" => FALSE, "menuWidth" => 120, "width" => 55));
		$this->add_MenuLink(Array("name" => "Modules", "exe" => FALSE, "menuWidth" => 130, "width" => 55));
        //$this->add_MenuLink(Array("name" => "Help", "exe" => FALSE, "menuWidth" => 120, "width" => 55));

		// Building Static Menu Items
        $this->add_MenuItem(Array("name" => "Summary", "link" => "main.location.href='summary.php';", "SEP" => FALSE, "SEC" => _USER), "ROOT/Go");
        $this->add_MenuItem(Array("name" => "Monitor", "link" => "main.location.href='view_group.php';", "SEP" => FALSE, "SEC" => _USER), "ROOT/Go");

		// MenuItems: Modules tree
		$this->add_MenuItem(Array("name" => "Add/Remove Module", "link" => "main.location.href='module_support.php';", SEC => 255), "ROOT/Modules");
		$this->add_MenuLink(Array("name" => "Installed_Modules", "exe" => FALSE, "menuWidth" => 120, "width" => FALSE, "SEP" => TRUE), "ROOT/Modules");
	}

	function add_MenuLink($link, $menu = "ROOT")
	{
		$loc = explode("/", $menu);
		$p = "this->menuItem";

		for($x=0;$x<sizeof($loc);$x++) {
			$p = $p . "[" . $loc[$x] . "]['menu']";
		}
		$name = $link['name'];
		$item = Array("exe" => $link['exe'], "menuWidth" => $link['menuWidth'], "width" => $link['width'], "type" => 1, "SEP" => $link['SEP']);

		eval("\$$p = array_merge_recursive(\$$p,Array(\"$name\" => \$item));");
	}
	
	function add_MenuItem($item, $menu = "ROOT")
	{
		$loc = explode("/", $menu);
		$p = "this->menuItem";

		for($x=0;$x<sizeof($loc);$x++) {
			$p = $p . "[" . $loc[$x] . "]['menu']";
		}
		$name = $item['name'];
		$link = ($item['link']) ? $item['link'] : FALSE;
		$SEP  = ($item['SEP']) ? $item['SEP'] : 0;
		$SEC  = ($item['SEC']) ? $item['SEC'] : _USER;
		eval("\$$p = array_merge_recursive(\$$p,Array(\"$name\" => Array(\"link\" => \"$link\", \"type\" => 0, \"SEP\" => \"$SEP\", \"SEC\" => \"$SEC\")));");
	}

	function create_Menu($xCord,$yCord)
	{
		$p = $this->menuItem['ROOT'];

		// Loading Final menu Items.
		$this->add_MenuItem(Array("name" => "Refresh", "link" => "window.location.reload();", "SEP" => FALSE, "SEC" => _USER), "ROOT/Go");
		$this->add_MenuItem(Array("name" => "Logout", "link" => "logoff = confirm('Do you wish to logoff?'); if(logoff) window.location.href='logoff.php';", "SEP" => TRUE, "SEC" => _USER), "ROOT/Go");

		// Loading CSS
		printf("<LINK rel=stylesheet href='css/menu/menu.css' type=text/css>\n");
		// Loading menu.js
		printf("<SCRIPT language='Javascript1.2' src='js/menu.js'></SCRIPT>\n");
		
		// Loading variables required by menu.js
		printf("<SCRIPT><!--\n");
		printf("    itemOnColor  = '%s';\n", $p['itemOnColor']);
		printf("    itemOffColor = '%s';\n", $p['itemOffColor']);
		printf("    menuNext     = \"%s\";\n", $p['menuNext']);
		printf("    newMenuXDepl = %s;\n", $p['newMenuXDepl']);
		printf("    newMenuYDepl = %s;\n", $p['newMenuYDepl']);
		printf("    menuBgColor  = '%s';\n", $p['menuBgColor']);
		printf("    menuFgColor  = '%s';\n", $p['menuFgColor']);
		printf("    im1 = new Image();\n");
		printf("    im1.src = 'images/menu/nimic.gif';\n");
		printf("    im2 = new Image();\n");
		printf("    im2.src = 'images/next.gif';\n");
		printf("	myStatus = 0;\n");
		
		echo $this->create_MenuItems(FALSE,FALSE,$xCord,$yCord);
		
		printf("	myStatus = 1;\n");
		printf("// --></SCRIPT>\n");
	}
	
	function create_MenuItems($start = FALSE, $loc = FALSE, $xCord = FALSE, $yCord = FALSE)
	{
		global $session, $color;
		// Building menu
		
		if($start) {
			$p = $start['menu'];                            // Using previous pointer
			$info = $start;                                 // menu configuration array
		} else {
			$p = $this->menuItem['ROOT']['menu'];           // Using root menu
			$info = $this->menuItem['ROOT'];                // menu configuration array
		}

		$js_code = NULL;

		if(is_array($p)) {
			foreach($p as $key => $value) {
				if($value['SEP'])
					$js_code .= sprintf("%s.addMenuSeparator(1);\n", $loc);

				if($value['type']) {
					if($loc) {
						$js_code .= sprintf("%s = %s.addMenu(1, \"<nobr>&nbsp;&nbsp;<b>%s</b></nobr>\", \"%s\", %d, %d);\n",
						            $key, $loc, $key, $info['exe'], $info['menuWidth'], $info['width']);
					} else {
						if($loaded) {
							$js_code .= sprintf("%s = addMenu(1, \"<nobr><b>%s</b></nobr>\", \"%s\", %d, %d);\n",
							            $key, $key, $value['exe'], $value['menuWidth'], $value['width']);
						} else {
							$js_code .= sprintf("%s = addMenu(1, \"<nobr><b>%s</b></nobr>\", \"%s\", %d, %d, %d, %d);\n",
							            $key, $key, $info['exe'], $value['menuWidth'], $value['width'], $xCord, $yCord);
							$loaded = TRUE;
						}
					}
					$js_code .= $this->create_MenuItems($value,$key);
				} else {
					if($session->checkAccessLevel($value['SEC']))
					    if($value['link'])
							$js_code .= sprintf("%s.addMenuItem(\"<nobr>&nbsp;&nbsp;<b>%s</b></nobr>\", \"%s\");\n", $loc, $key, $value['link']);
						else
						    $js_code .= sprintf("%s.addMenuItem(\"<nobr>&nbsp;&nbsp;<font color='%s'>%s</font></nobr>\", \"\");\n", $loc, $color[15], $key);
					else
						$js_code .= sprintf("%s.addMenuItem(\"<nobr>&nbsp;&nbsp;<strike>%s</strike></nobr>\", \"javascript:alert('Access denied');\");\n", $loc, $key);
				}
			}
		}

		return $js_code;
	}
}

// External Program Functions
function add_MenuLink($name, $width, $menuWidth, $menu = "ROOT", $exe = FALSE)
{
	global $_TOP_MENU;
	
	if(isset($_TOP_MENU)) {
		$link = Array("name" => $name, "width" => $width, "menuWidth" => $menuWidth, "exe" => $exe);
		$_TOP_MENU->add_MenuLink($link, $menu);
	}
}

function add_MenuItem($name, $link, $menu, $SEP = FALSE, $SEC = _USER)
{
	global $_TOP_MENU;

	if(isset($_TOP_MENU)) {
		$item = Array("name" => $name, "link" => $link, "SEP" => $SEP, "SEC" => $SEC);
		$_TOP_MENU->add_MenuItem($item, $menu);
	}
}

$_TOP_MENU = new menu();
?>
