<?
/********************************************************************
Copyright (c) 2003 Jason Sessler

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
********************************************************************/

/********************************************************************
	Changelog:
	11/30/2003  -   Fixed incorrect legend size when using more than
	                21 items in a graph.
				-   Changed the increment number on the automatic
				    color generation function. This function will
				    need to be rewritten.
				    
********************************************************************/
CLASS graph
{
	var $img;
	var $type;
	var $fontSize;
	var $saveAs;
	var $width;
	var $height;
	var $bx1;
	var $by1;
	var $bx2;
	var $by2;
	var $tmp_colors;
	var $colors;
	var $label;
	var $legend;
	var $data;
	
	function graph($type = FALSE, $width = FALSE, $height = FALSE, $saveAs = FALSE)
	{
		$this->type = $type;
		$this->width = $width;
		$this->height = $height;
		$this->saveAs = $saveAs;
		$this->bx1 = 80;
		$this->by1 = 20;
		$this->bx2 = 50;
		$this->by2 = 40;
	}
	
	function add_graph_data($label, $legendLabel, $data)
	{
		$this->data[$label][$legendLabel] = $data;
		$this->legend[$legendLabel] = TRUE;
	}
	
	function add_color_option($r, $g, $b)
	{
		$this->tmp_colors[sizeof($this->tmp_colors)] = Array("R" => $r, "G" => $g, "B" => $b);
	}

	function build_line_graph()
	{
		// Getting total image size
		$baseGraphWidth  = 400;
		$baseGraphHeight = 100;
		$legendGraphHeight = (sizeof($this->legend)*12);
		$width  = $baseGraphWidth+($this->bx1+$this->bx2);
		$height = $baseGraphHeight+$legendGraphHeight+($this->by1+$this->by2);

		$graphX = $baseGraphWidth+$this->bx1;
		$graphY = $baseGraphHeight+$this->by1;

		$this->img = imagecreate($width+1, $height+1);

		// Default colors and linestyles for graph
		$white = imagecolorallocate($this->img, 255, 255, 255);
		$black = imagecolorallocate($this->img, 0, 0, 0);
		$red = imagecolorallocate($this->img, 255, 0, 0);
		$gray  = imagecolorallocate($this->img, 127, 127, 127);

		$style  = array($gray, $gray, $white);
		$style2 = array($red, $red, $white);

		$this->build_color_options();

		// Building graph background
		imagerectangle($this->img, $this->bx1 ,$this->by1, $graphX, $graphY, $gray);
        imagerectangle($this->img, 0 ,0, $width-1, $height-1, $black);
        imagerectangle($this->img, 1 ,1, $width-2, $height-2, $gray);
        
		// Building horizontal reference lines
		$size = sizeof($this->data);
		$midPoint = ($size/2)-1;
		$step = $baseGraphWidth/$size;
		$xPos = ($baseGraphWidth+$this->bx1)-2;
		$hpx = $xPos;

		@reset($this->data);
		for($x=0;$x<$size;$x++) {
			$key = @key($this->data);
			if($x == 0 || $x == $midPoint || $x == $size-1 ) {
				imagestring($this->img, 2, $xPos-((strlen($key)*2)+4), $graphY+2, $key, $black);
				imagesetstyle($this->img, $style2);
				imageline($this->img, $xPos, $this->by1-2, $xPos, $this->by2+4, $red);
				 imageline($this->img, $xPos, $this->by1+1, $xPos, $graphY-1, IMG_COLOR_STYLED);
				imageline($this->img, $xPos, $graphY-2, $xPos, $graphY+4, $red);
			} else {
				imagesetstyle($this->img, $style);
				imageline($this->img, $xPos, $this->by1-2, $xPos, $this->by2+4, $gray);
				 imageline($this->img, $xPos, $this->by1+1, $xPos, $graphY-1, IMG_COLOR_STYLED);
				imageline($this->img, $xPos, $graphY-2, $xPos, $graphY+4, $gray);
			}
			$xPos-=$step;
			@next($this->data);
		}
		
		// Building vertical reference lines
		@reset($this->data);
		$x = 0;
		$avg = 0;
		if(is_array($this->data)) {
			foreach($this->data as $label) {
				if(is_array($label)) {
					foreach($label as $legendLabel => $legendValue) {
						if(is_array($legendValue)) {
							foreach($legendValue as $point) {
								if(!is_null($point)) {
									$item[$x] = $point;
									$legend[$legendLabel][] = $item[$x];
									$legendAvg[$legendLabel] += $item[$x];
									$avg+=$item[$x];
									$x++;
								}
							}
						}
					}
				}
			}
		}
		@rsort($item);

		$tp = round($item[0]*100)/100;
		$lp = round($item[sizeof($item)-1]*100)/100;

		$vpy['TOP'] = $this->by1;
		$vpy['LOW'] = ($graphY);
		
		if ($tp == 0 && $lp == 0) {
			$tp = NULL;
			//$lp = NULL;
		} else {
			if(($tp-$lp)<$lp) {
		    	$tp+=round(($tp/2)*100)/100;
		    	$lp-=round(($lp/2)*100)/100;
			}
			$ap = round((($tp+$lp)/2)*100)/100;
			$vpy['MID'] = ($vpy['LOW']+$vpy['TOP'])/2;
			$yStep = $baseGraphHeight/($tp-$lp);
		
			$aHp = round((($tp+$ap)/2)*100)/100;
			$vpy['MIDHIGH'] = ($vpy['MID']+$vpy['TOP'])/2;

			$aLp = round((($lp+$ap)/2)*100)/100;
			$vpy['MIDLOW'] = round(($vpy['LOW']+$vpy['MID'])/2);
		}
		// Top reference
		if(!is_null($tp)) {
			imagestring($this->img, 2, ($this->bx1-5)-(strlen($tp)*6), $vpy['TOP']-10, $tp, $black);
			imageline($this->img, $this->bx1-2, $vpy['TOP'], ($baseGraphWidth+$this->bx1)+4, $vpy['TOP'], $red);
			imagesetstyle($this->img, $style2);
			imageline($this->img, $this->bx1+1, $vpy['TOP'], ($baseGraphWidth+$this->bx1)-1, $vpy['TOP'], IMG_COLOR_STYLED);
		}
		// Middle High reference
		if(!is_null($aHp)) {
			imagestring($this->img, 2, ($this->bx1-5)-(strlen($aHp)*6), $vpy['MIDHIGH']-10, $aHp, $black);
			imageline($this->img, $this->bx1-2, $vpy['MIDHIGH'], ($baseGraphWidth+$this->bx1)+4, $vpy['MIDHIGH'], $red);
			imagesetstyle($this->img, $style2);
			imageline($this->img, $this->bx1+1, $vpy['MIDHIGH'], ($baseGraphWidth+$this->bx1)-1, $vpy['MIDHIGH'], IMG_COLOR_STYLED);
		}
		// Middle reference
		if(!is_null($ap)) {
			imagestring($this->img, 2, ($this->bx1-5)-(strlen($ap)*6), $vpy['MID']-10, $ap, $black);
			imageline($this->img, $this->bx1-2, $vpy['MID'], ($baseGraphWidth+$this->bx1)+4, $vpy['MID'], $red);
			imagesetstyle($this->img, $style2);
			imageline($this->img, $this->bx1+1, $vpy['MID'], ($baseGraphWidth+$this->bx1)-1, $vpy['MID'], IMG_COLOR_STYLED);
		}
		// Middle Low reference
		if(!is_null($aLp)) {
			imagestring($this->img, 2, ($this->bx1-5)-(strlen($aLp)*6), $vpy['MIDLOW']-10, $aLp, $black);
			imageline($this->img, $this->bx1-2, $vpy['MIDLOW'], ($baseGraphWidth+$this->bx1)+4, $vpy['MIDLOW'], $red);
			imagesetstyle($this->img, $style2);
			imageline($this->img, $this->bx1+1, $vpy['MIDLOW'], ($baseGraphWidth+$this->bx1)-1, $vpy['MIDLOW'], IMG_COLOR_STYLED);
		}
		// Low reference
		if(!is_null($lp)) {
			imagestring($this->img, 2, ($this->bx1-5)-(strlen($lp)*6), $vpy['LOW']-10, $lp, $black);
			imageline($this->img, $this->bx1-2, $vpy['LOW'], ($baseGraphWidth+$this->bx1)+4, $vpy['LOW'], $red);
			imagesetstyle($this->img, $style2);
			imageline($this->img, $this->bx1+1, $vpy['LOW'], ($baseGraphWidth+$this->bx1)-1, $vpy['LOW'], IMG_COLOR_STYLED);
		}
		// Building graph

		if(is_array($this->data)) {
			foreach($this->data as $label) {
				if(is_array($label)) {
					foreach($label as $legendLabel => $legendValue) {
						if(is_array($legendValue)) {
							if(is_null($xStep[$legendLabel])) {
								$xStep[$legendLabel] = $hpx;
							}
							$size = sizeof($legendValue);
							$xStepSize = $step/$size;
							@ksort($legendValue);
							@reset($legendValue);

							foreach($legendValue as $curStat) {
								if(!$item[$legendLabel]) {
									$xStep2[$legendLabel] = $xStep[$legendLabel];
								} else {
									$xStep[$legendLabel]-=$xStepSize;
									$xStep2[$legendLabel] = $xStep[$legendLabel]+$xStepSize;
								}

								if(is_null($prevStat[$legendLabel]))
									$prevStat[$legendLabel] = $curStat;

								if(!is_null($curStat) && $xStep[$legendLabel] > $this->bx1) {
									imageline($this->img, $xStep2[$legendLabel], $graphY-($prevStat[$legendLabel]-$lp)*$yStep, $xStep[$legendLabel], $graphY-($curStat-$lp)*$yStep, $this->colors[$legendLabel]);
								}
								$item[$legendLabel]++;
								$prevStat[$legendLabel] = $curStat;
							}
						}
					}
				}
			}
		}
		
		// Building Legend
		$yOff=20;
		if(is_array($this->legend)) {
			while(!is_null($key = @key($this->legend))) {
				imagefilledrectangle($this->img, $this->bx1 ,$graphY+$yOff, $this->bx1+8, ($graphY+$yOff)+8, $this->colors[$key]);
				imagerectangle($this->img, $this->bx1 ,$graphY+$yOff, $this->bx1+8, ($graphY+$yOff)+8, $black);
				$current = round($legend[$key][0]*100)/100;
				@rsort($legend[$key]);
				$legendSize = sizeof($legend[$key]);
				$high = round($legend[$key][0]*100)/100;
				$low  = round($legend[$key][$legendSize-1]*100)/100;
				$avg  = @round(($legendAvg[$key]/$legendSize)*100)/100;
				$string = sprintf("%s  Cur: %s     Low: %s Avg: %s High: %s", $key, $current, $low, $avg, $high);
				imagestring($this->img, 1, $this->bx1+15 ,$graphY+$yOff, $string, $black);
				$yOff+=12;
				@next($this->legend);
			}
		}
		if($this->saveAs) {
			imagepng($this->img, $this->saveAs);
		} else {
			imagepng($this->img);
		}
	}

	function build_color_options()
	{
		if(is_null($this->tmp_colors)) {
			$offset = 255;
			@reset($this->data);
			$x=1;
			if(!is_null($key = @key($this->data)) && $this->data) {
				while(!is_null($legendLabel = @key($this->data[$key]))) {
					if(!is_float($x/3)) {
						$this->colors[$legendLabel] = imagecolorallocate($this->img, $offset, 0, 0);
						$offset-=30;
						$x=0;
					} else if(!is_float($x/2)) {
						$this->colors[$legendLabel] = imagecolorallocate($this->img, 0, $offset, 0);
					} else if(!is_float($x/1)) {
						$this->colors[$legendLabel] = imagecolorallocate($this->img, 0, 0, $offset);
					}
					@next($this->data[$key]);
					$x++;
				}

			}
		} else {
			@reset($this->data);
			$x=0;
			while(!is_null($key = @key($this->data)) && $this->data) {
				$this->colors[$key] = imagecolorallocate($this->img, $this->tmp_colors[$x][0], $this->tmp_colors[$x][1], $this->tmp_colors[$x][2]);
				$x++;
				@next($this->data);
			}
		}
	}
}



?>
