<?
/********************************************************************
Copyright (c) 2003 Jason Sessler

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
********************************************************************/

/********************************************************************
	Changelog:
	11/30/2003  -   Cleaned up duplicate functions found in the
					idsconfig module and this module. Originally
					there was to be just one module.

	11/27/2003  -   Removed the LEFT JOIN from the IDS_EVENTS SELECT.
	                Instead of doing the LEFT JOIN to get the HostName
	                from the HOSTS table, we are doing a separate SELECT
	                query on the HOSTS table for any Hosts that have the
	                IsSniffer Field toggled on. Sped up the IDS Events
	                window over 65% on large IDS Event tables.
	                (+/-238000 Events)  PRE: ~20 secs to load.
	                                    POST: ~6 secs to load.
				-   Forgot to add the MIT License header.

********************************************************************/

function php_whois($host, $srvList)
{
	$parseList = explode(",", $srvList);
	$CONN = FALSE;
	for($x=0;$x<sizeof($parseList);$x++) {
		$socket = fsockopen(gethostbyname($parseList[$x]),43,$errno,$errmsg,30);
		if($socket) {
			$whoisSrv = $parseList[$x];
			// Getting WHOIS on HostName
			$sendData = sprintf("%s\n", $host);
			fputs($socket, $sendData, strlen($sendData));
			$whoisData = null;
			while($recvData = fgets($socket, 2048)) {
				$whoisData .= $recvData;
				if(strpos($recvData, "Whois Server") !== FALSE) {
					$refered = explode(":", $recvData);
					$referTo = trim($refered[1]);
				}
			}
			$CONN = TRUE;
			break;
			fclose($socket);
		} else {
			$whoisError .= sprintf("[%s]: (%s) %s\n", $parseList[$x], $errno, $errmsg);
		}
	}

	if(!$CONN) {
		$whoisSrv  = "NONE";
	    $whoisData = "ERROR: Unable to connect to WHOIS server(s)\n\n" . $whoisError;
	}
	if($referTo) {
		$tmp_whois = php_whois($host,$referTo);
		$whoisData .= sprintf("\r\nRefered To: [%s]\r\n\r\n%s", $referTo, $tmp_whois['DATA']);
	}

	return Array('SERVER' => $whoisSrv, 'DATA' => $whoisData);
}

function ids_css()
{
	printf("<LINK rel=stylesheet href='%s/modules/idsview/ids.css' type='text/css'>\n", _DOC_ROOT);
	return TRUE;
}

function ids_build($tableSize, $search = FALSE)
{
	global $db, $args, $idsShow, $idsStart, $color, $HID, $idsgraph, $_SERVER;

	if(function_exists(graphing)) {
		$graph = graphing(Array("type" => "SNORT", "HID" => $HID, "graphType" => "idsgraph"), $idsgraph);
	} else {
		$graph = FALSE;
	}

	if($db->dbc) {
		$ids = new conn_mySQL();
		$ids->db_use_conn($db->dbc, $db->db);
		$closeit = FALSE;
	} else {
		global $credentials;
		$ids = new conn_mySQL($credentials);
		$ids->db_new_conn();
		$closeit = TRUE;
	}

	if(!$search) {
		if(isset($HID)) {
		    $ids->add_table(_TABLE_HOSTS);
		    $ids->add_select_item("IPAddr");
		    $ids->clause = "where HID='$HID'";
		    $ret = $ids->compile_query("SELECT");
		    if($ret['return']) {
		    	$ids->run_query();
				if($ids->rows > 0) {
			    	$ids->step();
			    	$where = sprintf("where SIP='%s' OR DIP='%s'", $ids->res['IPAddr'], $ids->res['IPAddr']);
			    } else {
			    	return "No matching Hosts HID($HID)";
			    }
			} else {
				return $ret['errmsg'];
			}
		} else {
			$IPAddr = null;
		}
	} else {
		$where = "where $search";
	}

	$ids->clear_res();

	if(is_null($idsShow))
		$idsShow = 5;
	if(is_null($idsStart))
	    $idsStart = 0;
	    
    $ids->add_table(_TABLE_IDS_EVENTS);
    $ids->add_select_item("count(EID) as num");

	$ids->clause = $where;

	$ret = $ids->compile_query("SELECT");
	if($ret['return']) {
		$ids->run_query();
		$ids->step();
		$totalSize = $ids->res['num'];
	}

	$ids->clear_res();

	$ids->add_table(_TABLE_HOSTS);
	$ids->add_select_item(Array("HID", "HostName"));
	$ids->clause = "where IsSniffer='1'";
	$ids->add_extra("ORDER by HID");
	$ret = $ids->compile_query("SELECT");
	if($ret['return']) {
		$ids->run_query();
		for($x=0;$x<$ids->rows;$x++) {
			$ids->step();
			$Sniffer[$ids->res['HID']] = $ids->res['HostName'];
		}
	}
	$ids->clear_res();

	$ids->add_table(_TABLE_IDS_EVENTS);
	$ids->add_select_item(Array(_TABLE_IDS_EVENTS => Array("EID", "HID", "SIP", "SPort", "DIP", "DPort", "Protocol", "Descr", "TimeStamp")));

	$ids->clause = $where;

	$ids->add_extra("Order by TimeStamp DESC");
	$ids->add_extra("LIMIT $idsStart, $idsShow");
	$ret = $ids->compile_query("SELECT");
	if($ret['return']) {
		$ids->run_query();
		for($x=0;$x<$ids->rows;$x++) {
			$ids->step();
			$items .= html_tag("TR",
			              html_tag("TD", sprintf("<A class='ids' HREF='%s/modules/idsview/ids.php?LOC=view&EID=%s'>%s</A>\n", _DOC_ROOT, $ids->res['EID'], $ids->res['Descr']), $color[12], "align=left valign=top class=blacksmall") .
			              html_tag("TD", sprintf("<nobr><A class='ids' HREF='%s/modules/idsview/ids.php?LOC=search&Protocol=%s'>%s</A></nobr>\n", _DOC_ROOT, $ids->res['Protocol'], $ids->res['Protocol']), $color[12], "align=center valign=top class=blacksmall") .
			              html_tag("TD", sprintf("<nobr><A class='ids' HREF='%s/modules/idsview/ids.php?LOC=search&SIP=%s'>%s</A>:<A class='ids' HREF='%s/modules/idsview/ids.php?LOC=search&SPort=%s'>%s</A></nobr>\n",
						  	  _DOC_ROOT, $ids->res['SIP'], $ids->res['SIP'], _DOC_ROOT, $ids->res['SPort'], $ids->res['SPort']),
						  $color[12], "align=left valign=top class=blacksmall") .
			              html_tag("TD", sprintf("<nobr><A class='ids' HREF='%s/modules/idsview/ids.php?LOC=search&DIP=%s'>%s</A>:<A class='ids' HREF='%s/modules/idsview/ids.php?LOC=search&DPort=%s'>%s</A></nobr>\n",
						  	  _DOC_ROOT, $ids->res['DIP'], $ids->res['DIP'], _DOC_ROOT, $ids->res['DPort'], $ids->res['DPort']),
						  $color[12], "align=left valign=top class=blacksmall") .
			              html_tag("TD", sprintf("<nobr><A class='ids' HREF='%s/modules/idsview/ids.php?LOC=search&HID=%s'>%s</A></nobr>\n", _DOC_ROOT, $ids->res['HID'], $Sniffer[$ids->res['HID']]), $color[12], "align=center valign=top class=blacksmall") .
						  html_tag("TD", sprintf("<nobr>%s</nobr>\n", $ids->res['TimeStamp']), $color[12], "align=center valign=top class=blacksmall"),
					  "", "");
		}
	} else {
		$items = html_tag("TR",
		             html_tag("TD", $ret['errmsg'], $color[3], "colspan=5 align=center class=errmsg"),
				 "", "");
	}
	$showList = Array("1"   => 1, "5"   => 5, "10"  => 10, "20"  => 20,
					  "30"  => 30, "40"  => 40, "50"  => 50, "100" => 100);

	if($items) {
		parsePost(Array("LOC", "Descr", "Protocol", "SIP", "SPort", "DIP", "DPort", "HID"));

		$table = html_tag("TABLE",
		             (($graph) ? $graph : "") .
				 	 html_tag("TR",
				     	 html_tag("TD", "<b>IDS Events History</b>\n", $color[9], "class=blacksmall align=center"),
				 	 "", "") .
				 	 html_tag("TR",
				     	 html_tag("TD",
				         	 sprintf("<FORM method=get action='%s'>\n", $_SERVER['PHP_SELF']) .
						 	 sprintf("<SPAN class='blacksmall'><b>Show Last %s Event(s)</b></SPAN>&nbsp;&nbsp;<input type=submit value='Go'>\n",
							 	 build_select("idsShow", $showList, $idsShow)) .
						 	 buildHiddenValues("idsShow") .
						 	 sprintf("</FORM>\n"),
					 	 $color[3], "colspan=5 class=blacksmall align=center"),
				 	 "", "") .
				 	 html_tag("TR",
				 	 	 html_tag("TD",
					     	 html_tag("TABLE",
						     	 html_tag("TR",
							     	 html_tag("TD", "<nobr><b>Signature</b></nobr>\n", $color[0], "class='blacksmall' align=center") .
								 	 html_tag("TD", "<nobr><b>Type</b></nobr>\n", $color[0], "class='blacksmall' align=center") .
								 	 html_tag("TD", "<nobr><b>Source</nobr>\n", $color[0], "class='blacksmall' align=center") .
								 	 html_tag("TD", "<nobr><b>Destination</b></nobr>\n", $color[0], "class='blacksmall' align=center") .
								 	 html_tag("TD", "<nobr><b>Sensor</b></nobr>\n", $color[0], "class='blacksmall' align=center") .
								 	 html_tag("TD", "<nobr><b>Date/Time</b></nobr>\n", $color[0], "class='blacksmall' align=center"),
							 	 "", "") .
							 	 $items,
						 	 $color[10], "width=100% cellspacing=1 cellpadding=3"),
					 	 $color[3], ""),
				 	 "", "") .
				 	 html_tag("TR",
				 	 	 html_tag("TD", pageinate($_SERVER['PHP_SELF'], $totalSize, Array('Start' => Array('idsStart' => $idsStart), 'Size' => Array('idsShow' => $idsShow))),
						   	 $color[3], ""),
					 "", ""),
			 	 $color[10], "width=$tableSize, cellspacing=1 cellpadding=1 align=center");
	} else {
		$table = html_tag("TABLE",
		             (($graph) ? $graph : "") .
				 	 html_tag("TR",
				     	 html_tag("TD", "<b>IDS Events History</b>\n", $color[9], "class=blacksmall align=center"),
				 	 "", "") .
				 	 html_tag("TR",
				 	 	 html_tag("TD", "No Events Found\n", $color[3], "class=blacksmall align=center"),
				 	 "", ""),
			 	 $color[10], "width=$tableSize, cellspacing=1 cellpadding=1 align=center");
	}

	$ids->clear_res();
	if($closeit)
	    $ids->db_close();

	$ids = null;
	return $table;
}

?>
