/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASa
*       Filename : snpp.c
*		Handles the SNPP session when send a pager alert.
*
*       Updated :
*               11/06/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <string.h>

#include "../Common/Commoninclude.h"

#include "includes.h"
#include "extern.h"

/******************************************************************************
*       int SNPP_SendPage(char * PagerID, char * PagerMsg)
*		Handles the SNPP session when sending a pager alert.
*
*	Args :
*		char * PagerID		The Pager ID to send page to.
*		char * PagerMsg		The Pager message to send.
*
*       Updated :
******************************************************************************/
int SNPP_SendPage(char * PagerID, char * PagerMsg)
{
	int sock;
	int connected = 0;
	int Status = 0;
	char * ReadBuff = 0;
	char * WriteBuff = 0;
	struct sockaddr_in addr;
	struct hostent * hostinfo = 0;
		
	MyDebug(__LINE__, __FILE__, LOG_SMTP, "[SNPP_SendPage]-[Sending Page to :%s]-[Server : %s:%s]", 
					      PagerID, ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.PAGER.SERVER"),
					      ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.PAGER.SERVER.PORT"));
					      
	sock = socket(AF_INET, SOCK_STREAM, 0);
    	
    	hostinfo = gethostbyname(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.PAGER.SERVER"));
    	
    	addr.sin_family = AF_INET;
    	addr.sin_port = htons(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.PAGER.SERVER.PORT")));
	addr.sin_addr = *(struct in_addr *) *hostinfo->h_addr_list;
	memset(&(addr.sin_zero), '\0',8);

    	bzero(&(addr.sin_zero), 8); 
    
    	connected = connect(sock, (struct sockaddr *)&addr, sizeof(struct sockaddr));
    
    	if(connected != -1)
    	{
		MyDebug(__LINE__, __FILE__, LOG_SMTP, "[SNPP_SendPage]-[Connected]");
		
    		ReadBuff = RecvFromSocket(sock, "\n", 0);
    		ReadBuff = dfree(ReadBuff);

  		WriteBuff = dsprintf(WriteBuff, "PAGE %s", PagerID);
  		SendToSocket(sock, WriteBuff, "\r\n", 0);
		WriteBuff = dfree(WriteBuff);
		
	    	ReadBuff = RecvFromSocket(sock, "\n", 0);
	    	ReadBuff = dfree(ReadBuff);

		WriteBuff = dsprintf(WriteBuff, "MESS %s", PagerMsg);
		SendToSocket(sock, WriteBuff, "\r\n", 0);
		WriteBuff = dfree(WriteBuff);
		
		ReadBuff = RecvFromSocket(sock, "\n", 0);
		ReadBuff = dfree(ReadBuff);
	
		SendToSocket(sock, "SEND", "\r\n", 0);

		ReadBuff = RecvFromSocket(sock, "\n", 0);
		ReadBuff = dfree(ReadBuff);

		SendToSocket(sock, "QUIT", "\r\n", 0);

		Status = 1;
	}
	
	MyDebug(__LINE__, __FILE__, LOG_SMTP, "[SNPP_SendPage]-[Exit]-[Status:%d]", Status);
	
	close(sock);

	return(Status);
}

