/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASc
*       Filename : check_local.c
*		Functions for performing local (Disk, CPU, Ram, etc) checks.
*
*       Updated :
*               11/07/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef __sun
#include <sys/loadavg.h>
#endif

#include "../Common/Commoninclude.h"
#include "includes.h"
#include "extern.h"

/******************************************************************************
*       char * Check_Local_Disk(struct XMLLeaf * RootLeaf)
*		Performs disk check. Uses the command specified in MIDASc.cf's
*		CONFIG.CMD.DISK. 
*
*	Args :
*		struct XMLLeaf * RootLeaf	XML linked list of Check config
*						options.
*
*       Updated :
******************************************************************************/
char * Check_Local_Disk(struct XMLLeaf * RootLeaf)
{
	struct XMLLeaf * LocalLeaf = 0;
	struct XMLLeaf * CheckLeaf = 0;
	
	char * PARTITION = 0;
	int PER_FREE = 0;
	int PER_USED = 0;
	int Per_Free = 0;
	int Per_Used = 0;
	char * Part = 0;
	unsigned SPACE_FREE = 0;
	unsigned SPACE_USED = 0;
	unsigned Space_Total = 0;
	unsigned Space_Used = 0;
	unsigned Space_Free = 0;
	char LINE[255];
	char * CMD = 0;
	FILE * pp;
	int Status = 1;
	int PartStatus = 0;
	char * Detail = 0;
	char * tDetail = 0;
	char * tResponse = 0;
	char * Response = 0;
	char * STAT = 0;
	char * tSTAT = 0;
	int OptionCount = 0;
			
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Local_Disk]-[Enter]");
	
	Response = dsprintf(Response, "<CHECK><HID>%s</HID><CID>%s</CID>", 
				    ReturnXMLLeafValue(RootLeaf, "SERVER.RETURNCHECKS.CHECK.HID"),
				    ReturnXMLLeafValue(RootLeaf, "SERVER.RETURNCHECKS.CHECK.CID"));
	LocalLeaf = RootLeaf;				
	CheckLeaf = RootLeaf;

	while(LocalLeaf)
	{
		CheckLeaf = BuildSubXMLTree(LocalLeaf, ReturnXMLLeafID(LocalLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.DISK"), 1);
		if(CheckLeaf)
		{
			PARTITION = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.DISK.PARTITION");
			PER_FREE = Myatoi(ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.DISK.PERFREE"));
			PER_USED = Myatoi(ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.DISK.PERUSED"));
			SPACE_FREE = Myatoll(ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.DISK.SPACEFREE"));
			SPACE_USED = Myatoll(ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.DISK.SPACEUSED"));
							      
			if(PARTITION)
			{
				
				
				MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Local_Disk]-[Partition:%s]-[Per:%d/%d]-[Space:%u/%u]", 
							      PARTITION, PER_FREE, PER_USED, SPACE_FREE, SPACE_USED);
			
				CMD = dsprintf(CMD, "%s %s", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.CMD.DISK"), PARTITION);
				MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Local_Disk]-[CMD:%s]", CMD);
				
				pp = popen(CMD,"r");
				if(pp)
				{
					fgets(LINE, 255, pp);
					do
					{
						PartStatus = 1;
						fgets(LINE, 255, pp);
						if(!feof(pp))
						{
							Part = strtok(LINE, " ");
							Space_Total = Myatoll(strtok(NULL, " "));
							Space_Used = Myatoll(strtok(NULL, " "));
							Space_Free = Myatoll(strtok(NULL, " "));
							Per_Used = Myatoi(strtok(NULL, " "));
							Per_Free = 100 - Per_Used;
							
							MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Local_Disk]-[Part:%s]-[Space:%u/%u/%u]-[Per:%d/%d]",
											       Part, Space_Total, Space_Used, Space_Free, Per_Used, Per_Free);
							if(SPACE_USED && Space_Used >= SPACE_USED)
							{
								PartStatus = 0;
							}
							
							if(SPACE_FREE && Space_Free <= SPACE_FREE)
							{
								PartStatus = 0;
							}
							
							if(PER_USED && Per_Used >= PER_USED)
							{
								PartStatus = 0;
							}
							
							if(PER_FREE && Per_Free <= PER_FREE)
							{
								PartStatus = 0;
							}
							
							if(PartStatus)
							{
								tDetail = dsprintf(tDetail, "[GREEN] %s Total:%u Used:%u(%d%%) Free:%u(%d%%)\n",
											     Part, Space_Total, Space_Used, Per_Used, Space_Free, Per_Free);
								Detail = dstrcat(Detail, tDetail);
								tDetail = dfree(tDetail);
							} else {
								tDetail = dsprintf(tDetail, "[RED] %s Total:%u Used:%u(%d%%) Free:%u(%d%%)\n",
											     Part, Space_Total, Space_Used, Per_Used, Space_Free, Per_Free);
								Detail = dstrcat(Detail, tDetail);
								tDetail = dfree(tDetail);
								Status = 0;
							}
							
							tSTAT = dsprintf(tSTAT, "<STAT><TYPE>CHECK</TYPE><ID>%s_%s</ID><ORDER>%d</ORDER><LABEL>%s</LABEL><DATAVAL>%u</DATAVAL></STAT>",
				    						ReturnXMLLeafValue(RootLeaf, "SERVER.RETURNCHECKS.CHECK.CID"),
				    						ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),
				    						OptionCount, Part, Space_Free);
							STAT = dstrcat(STAT, tSTAT);
							tSTAT = dfree(tSTAT);
							
							OptionCount++;
						}
					}while(!feof(pp));
				}
				pclose(pp);
				CMD = dfree(CMD);
				
				LocalLeaf = FindXMLLeafByID(LocalLeaf, CheckLeaf->ID);
			}
			CheckLeaf = DelSubXMLTree(CheckLeaf);
		} else {
			break;
		}
		
		if(LocalLeaf)
		{
			LocalLeaf = LocalLeaf->NextLeaf;
		}
	}
	
	if(!Detail)
	{
		Detail = dstrcpy(Detail, "No Checks to perform");
	}

	if(!STAT)
	{
		STAT = dstrcpy(STAT, "");
	}
		                                
	tResponse = dsprintf(tResponse, "<STATUS>%s</STATUS><DETAIL>%s</DETAIL>%s</CHECK>", STATUS[Status], Detail, STAT);
	Response = dstrcat(Response, tResponse);
	tResponse = dfree(tResponse);
	
	Detail = dfree(Detail);
	STAT = dfree(STAT);
	
	return(Response);
}

/******************************************************************************
*       char * Check_Local_CPU(struct XMLLeaf * CheckLeaf)
*		Performs CPU/Load check. Will either use the getloadavg()
*		function (if present on platform) or use the command specified 
*		in MIDASc.cf's CONFIG.CMD.CPU. 
*
*	Args :
*		struct XMLLeaf * RootLeaf	XML linked list of Check config
*						options.
*
*       Updated :
******************************************************************************/
char * Check_Local_CPU(struct XMLLeaf * CheckLeaf)
{
	double load[3];
	double MaxLoad;
	char * Response = 0;
	char * tResponse = 0;
	char * Detail = 0;
	char * tDetail = 0;
	int Status = 1;
	int i;
	char * STAT = 0;
		
	Response = dsprintf(Response, "<CHECK><HID>%s</HID><CID>%s</CID>", 
				    ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.HID"),
				    ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.CID"));
	
	if(CheckLeaf)
	{
		MaxLoad = Myatof(ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.CPU.MAXLOAD"));
	
		if(MaxLoad)
		{
#ifdef GETLOADAVG
			Detail = dstrcpy(Detail, "Load Avg (1m, 5m, 15m) : ");
			if(getloadavg(load, 3) != -1)
			{
				for(i=0;i<3;i++)
				{
					if(load[i] >= MaxLoad)
					{
						Status = 0;
					}
					tDetail = dsprintf(tDetail, ", %.2f", load[i]);
					Detail = dstrcat(Detail, tDetail);
					tDetail = dfree(tDetail);
				}
				STAT = dsprintf(STAT, "<STAT><TYPE>CHECK</TYPE><ID>%s_%s</ID><ORDER>0</ORDER><LABEL>CPU Load</LABEL><DATAVAL>%.2f</DATAVAL></STAT>",
								ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.CID"),
								ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),
								load[1]);
			}
#else 
			//
			//	[TCC] Some systems do not support the getloadavg function. In that case 
			//	      we do it the old fashion way. We pipe the results of the uptime
			//	      command and parse out the results.
			//
			FILE *pp;
			char * tLINE = 0;
			
			tDetail = 0;
					
			pp = popen(UPTIMECMD,"r");
                        if(pp)
                        {
                        	tLINE = drealloc(tLINE, 1024);
				fgets(tLINE,1023, pp);
                                tLINE = dstrclean(tLINE);
				if(tLINE)
                                {
                                	Detail = dstrcpy(Detail, tLINE);
                                        	
                                        strtok(tLINE, ":");
                                        strtok(NULL, ":");
                                        strtok(NULL, ":");
                                        strtok(NULL, " ");

                                        for(i=0;i<3;i++)
                                        {
                                        	load[i] = Myatof(strtok(NULL, " "));
						if(load[0] >= MaxLoad)
                                                {
                                                	Status = 0;
                                                }
					}
                                        
                                        STAT = dsprintf(STAT, "<STAT><TYPE>CHECK</TYPE><ID>%s_%s</ID><ORDER>0</ORDER><LABEL>CPU Load</LABEL><DATAVAL>%.2f</DATAVAL></STAT>",
								ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.CID"), 
								ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),
								load[1]);
				}
				tLINE = dfree(tLINE);
			}
#endif
		}
	}
	
	if(!Detail)
	{
		Detail = dstrcpy(Detail, "No Checks to perform");
	}

	if(!STAT)
	{
		STAT = dstrcpy(STAT, "");
	}
		
	tResponse = dsprintf(tResponse, "<STATUS>%s</STATUS><DETAIL>%s</DETAIL>%s</CHECK>", STATUS[Status], Detail, STAT);
	Response = dstrcat(Response, tResponse);
	tResponse = dfree(tResponse);
	
	STAT = dfree(STAT);
	Detail = dfree(Detail);
	
	return(Response);
}

/******************************************************************************
*       char * Check_Local_Proc(struct XMLLeaf * RootLeaf)
*		Performs process check. Uses the command specified in 
*		MIDASc.cf's CONFIG.CMD.PROC.
* 
*	Args :
*		struct XMLLeaf * RootLeaf	XML linked list of Check config
*						options.
*
*       Updated :
******************************************************************************/	
char * Check_Local_Proc(struct XMLLeaf * RootLeaf)
{
	struct XMLLeaf * LocalLeaf;
	struct XMLLeaf * CheckLeaf;
	char * Response = 0;
	char * tResponse = 0;
	char * ProcCmdString = 0;
	char * ProcName = 0;
	char * CheckType = 0;
	int Status = 1;
	int ProcCount = 0;
	int MaxCount = 0;
	char * Detail = 0;
	char * tDetail = 0;
	FILE *pp;
	char * STAT = 0;
	char * tSTAT = 0;
	
	Response = dsprintf(Response, "<CHECK><HID>%s</HID><CID>%s</CID>", 
				    ReturnXMLLeafValue(RootLeaf, "SERVER.RETURNCHECKS.CHECK.HID"),
				    ReturnXMLLeafValue(RootLeaf, "SERVER.RETURNCHECKS.CHECK.CID"));

	MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Local_Proc]-[Enter]");
	
	LocalLeaf = RootLeaf;
	CheckLeaf = RootLeaf;
	
	if(FindXMLLeafByTag(LocalLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.PROC"))
	{
		pp = popen(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.CMD.PROC"), "r");
		if(pp)
		{
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Local_Proc]-[CMD:%s]", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.CMD.PROC"));
	
			ProcCmdString = dfiletostring(pp);
			pclose(pp);
			
			if(ProcCmdString)
			{
				while(LocalLeaf)
				{
					CheckLeaf = BuildSubXMLTree(LocalLeaf, ReturnXMLLeafID(LocalLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.PROC"), 1);
					if(CheckLeaf)
					{
						CheckType = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.PROC.TYPE");
						if(CheckType)
						{
							MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Local_Proc]-[CheckType:%s]", CheckType);
							if(!strcasecmp(CheckType, "PRESENT"))
							{
								ProcName = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.PROC.NAME");
								if(!strstr(ProcCmdString, ProcName))
								{
									Status = 0;
									tDetail = dsprintf(tDetail, "[RED] %s not running\n", ProcName);
								} else {
									tDetail = dsprintf(tDetail, "[GREEN] %s running\n", ProcName);
								}
								Detail = dstrcat(Detail, tDetail);
								tDetail = dfree(tDetail);
							}
						
							if(!strcasecmp(CheckType, "ABSENT"))
							{
								ProcName = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.PROC.NAME");
								if(!strstr(ProcCmdString, ProcName))
								{
									tDetail = dsprintf(tDetail, "[GREEN] %s not running\n", ProcName);
								} else {
									Status = 0;
									tDetail = dsprintf(tDetail, "[RED] %s running\n", ProcName);
								}
								Detail = dstrcat(Detail, tDetail);
								tDetail = dfree(tDetail);
							}
							
							if(!strcasecmp(CheckType, "COUNT"))
							{
								MaxCount = Myatoi(ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.PROC.LIMIT"));
								if(MaxCount)
								{
									
									ProcCount = Check_Local_Proc_Count(ProcCmdString, 0);
									if(ProcCount >= MaxCount)
									{
										Status = 0;
										tDetail = dsprintf(tDetail, "[RED] %d processes running, max is %d.\n", ProcCount, MaxCount);
									} else {
										tDetail = dsprintf(tDetail, "[GREEN] %d processes running, max is %d.\n", ProcCount, MaxCount);
									}
									Detail = dstrcat(Detail, tDetail);
									tDetail = dfree(tDetail);
								
									tSTAT = dsprintf(tSTAT, "<STAT><TYPE>CHECK</TYPE><ID>%s_%s</ID><ORDER>0</ORDER><LABEL>Process Count</LABEL><DATAVAL>%d</DATAVAL></STAT>",
				    								ReturnXMLLeafValue(RootLeaf, "SERVER.RETURNCHECKS.CHECK.CID"),
				    								ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),
				    								ProcCount);
									STAT = dstrcat(STAT, tSTAT);
									tSTAT = dfree(tSTAT);
								}
							}
					
							if(!strcasecmp(CheckType, "ZOMBIE"))
							{
								MaxCount = Myatoi(ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.PROC.LIMIT"));
								if(MaxCount)
								{
									ProcCount = Check_Local_Proc_Count(ProcCmdString, 1);
									if(ProcCount >= MaxCount)
									{
										Status = 0;
										tDetail = dsprintf(tDetail, "[RED] %d zombie processes running, max is %d.\n", ProcCount, MaxCount);
									} else {
										tDetail = dsprintf(tDetail, "[GREEN] %d zombie processes running, max is %d.\n", ProcCount, MaxCount);
									}
									Detail = dstrcat(Detail, tDetail);
									tDetail = dfree(tDetail);
									
									tSTAT = dsprintf(tSTAT, "<STAT><TYPE>CHECK</TYPE><ID>%s_%s</ID><ORDER>1</ORDER><LABEL>Zombie Count</LABEL><DATAVAL>%d</DATAVAL></STAT>",
				    						ReturnXMLLeafValue(RootLeaf, "SERVER.RETURNCHECKS.CHECK.CID"), 
				    						ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),
				    						ProcCount);
									STAT = dstrcat(STAT, tSTAT);
									tSTAT = dfree(tSTAT);
								}
							}	
						}
				
						LocalLeaf = FindXMLLeafByID(LocalLeaf, CheckLeaf->ID);
						CheckLeaf = DelSubXMLTree(CheckLeaf);
					}
					if(LocalLeaf)
					{
						LocalLeaf = LocalLeaf->NextLeaf;
					}
				}
			}
		}
	}
	
	if(!Detail)
        {
                Detail = dstrcpy(Detail, "No Checks to perform");
        }

	if(!STAT)
	{
		STAT = dstrcpy(STAT, "");
	}
	
	tResponse = dsprintf(tResponse, "<STATUS>%s</STATUS><DETAIL>%s</DETAIL>%s</CHECK>", STATUS[Status], Detail, STAT);
	Response = dstrcat(Response, tResponse);
	tResponse = dfree(tResponse);
	
	Detail = dfree(Detail);
	STAT = dfree(STAT);
	ProcCmdString = dfree(ProcCmdString);
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Local_Proc]-[Exit]");
			
	return(Response);
}

/******************************************************************************
*	int Check_Local_Proc_Count(char *pslist, int ZombieOnly)
*		Parses pslist and returns either the total number of processes
*		running or the number of zombie processes (if ZombieOnly is
*		TRUE).
*
*	Args :
*		char *pslist		String containing the output of ps.
*		int ZombieOnly		If true Count only zombies.
*
*       Updated :
******************************************************************************/	
int Check_Local_Proc_Count(char *pslist, int ZombieOnly)
{
	char * tLINE = 0;
	char * tSTATE = 0;
	int total = 0;
	char * Localpslist = 0;
	
	Localpslist = dstrcpy(Localpslist, pslist);
	do
	{
		tLINE = strsep(&Localpslist, "\n");
		if(tLINE)
		{
			if(strstr(tLINE, ":"))
			{
				strtok(tLINE," ");
				tSTATE = strtok(NULL, " ");
				if(tSTATE)
				{
					if(ZombieOnly)
					{
						if(!strcmp(tSTATE, "Z"))
						{
							total++;
						}
					} else {
						total++;
					}
				}
			}
		}
	}while(tLINE);
	Localpslist = dfree(Localpslist);
	
	return(total);
}

/******************************************************************************
*       char * Check_Local_Logs(struct XMLLeaf * RootLeaf)
*		Performs log check. Will check specified log for string matches
*		(using regex).
*
*	Args :
*		struct XMLLeaf * RootLeaf	XML linked list that contains 
*						check options.
*
*       Updated :
******************************************************************************/
char * Check_Local_Logs(struct XMLLeaf * RootLeaf)
{
	struct XMLLeaf * LocalLeaf;
	struct XMLLeaf * CheckLeaf;
	char * Response = 0;
	char * tResponse = 0;
	char * Detail = 0;
	char * tDetail = 0;
	char * LOGFILE = 0;
	char * LOGIC = 0;
	char * REGEX = 0;
	char * EXPECT = 0;
	char LINE[255];
	char * MD5 = 0;
	FILE *pp;
	char * CMD = 0;
	int Status = 1;
	unsigned LogLineCount = 0;
	char * STAT = 0;
	char * tSTAT = 0;
	unsigned NUMLINES = 0;
	int OptionCount = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Local_Logs]-[Enter]");

	
	
	Response = dsprintf(Response, "<CHECK><HID>%s</HID><CID>%s</CID>", 
				    ReturnXMLLeafValue(RootLeaf, "SERVER.RETURNCHECKS.CHECK.HID"),
				    ReturnXMLLeafValue(RootLeaf, "SERVER.RETURNCHECKS.CHECK.CID"));

	LocalLeaf = RootLeaf;
	CheckLeaf = RootLeaf;
	
	while(LocalLeaf)
	{
		CheckLeaf = BuildSubXMLTree(LocalLeaf, ReturnXMLLeafID(LocalLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.LOG"), 1);
		if(CheckLeaf)
		{
			
			
			LOGFILE = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.LOG.LOGFILE");
			LOGIC = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.LOG.LOGIC");
			REGEX = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.LOG.REGEX");
			EXPECT = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.LOG.EXPECT");
			NUMLINES = Myatoll(ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.LOG.NUMLINES"));
			
			if(!NUMLINES)
			{
				NUMLINES = 100;
			}
				
			if(LOGFILE && LOGIC && REGEX && EXPECT)
			{
				MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Local_Logs]-[LOGFILE:%s]-[LOGIC:%s]-[REGEX:%s]-[EXPECT:%s]-[NUMLINES:%u]"
									, LOGFILE, LOGIC, REGEX, EXPECT, NUMLINES);
					
				CMD = dsprintf(CMD, "%s -%u %s", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.CMD.LOG"), NUMLINES, LOGFILE);
				pp = popen(CMD,"r");
				CMD = dfree(CMD);
				if(pp)
				{
					LogLineCount = 0;
					do 
					{
						fgets(LINE, 255, pp);
						if(!feof(pp))
						{
							if(CheckStringLogic(LINE, REGEX, LOGIC, EXPECT))
							{
								MD5 = MD5String(LINE);
								if(Check_Local_Log_UniqueEntry(LOGFILE, MD5))
								{ 
									Status = 0;
									tDetail = dsprintf(tDetail, "[%s] - %s\n", LOGFILE, LINE);
									tDetail = CleanXMLValueString(tDetail);
									Detail = dstrcat(Detail, tDetail);
									tDetail = dfree(tDetail);
									
									LogLineCount++;
								}
								MD5 = dfree(MD5);
							}
						}
					}while(!feof(pp));
					fclose(pp);
					
					tSTAT = dsprintf(tSTAT, "<STAT><TYPE>CHECK</TYPE><ID>%s_%s</ID><ORDER>%d</ORDER><LABEL>%s</LABEL><DATAVAL>%u</DATAVAL></STAT>",
								ReturnXMLLeafValue(RootLeaf, "SERVER.RETURNCHECKS.CHECK.CID"),
								ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),
				    				OptionCount,
				    				LOGFILE,
				    				LogLineCount);
					STAT = dstrcat(STAT, tSTAT);
					tSTAT = dfree(tSTAT);
					
					OptionCount++;
				} else {
					Status = 0;
					tDetail = dsprintf(tDetail, "ERROR: cant open logfile : %s\n", LOGFILE);
					Detail = dstrcat(Detail, tDetail);
					tDetail = dfree(tDetail);
					MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Local_Logs]-[Unable to open logfile : %s", LOGFILE);
				}
			}											
			LocalLeaf = FindXMLLeafByID(LocalLeaf, CheckLeaf->ID);
			CheckLeaf = DelSubXMLTree(CheckLeaf);	
		}
		if(LocalLeaf)
		{
			LocalLeaf = LocalLeaf->NextLeaf;
		}	
	}
	
	if(!Detail)
	{
		Detail = dstrcpy(Detail, "[GREEN] No alarming log entries.");
	}
	
	if(!STAT)
	{
		STAT = dstrcpy(STAT, "");
	}
		
	tResponse = dsprintf(tResponse, "<STATUS>%s</STATUS><DETAIL>%s</DETAIL>%s</CHECK>", STATUS[Status], Detail, STAT);
	Response = dstrcat(Response, tResponse);
	tResponse = dfree(tResponse);

	STAT = dfree(STAT);
	Detail = dfree(Detail);
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[Check_Local_Logs]-[Exit]");
	return(Response);
}

/******************************************************************************
*       int Check_Local_Log_UniqueEntry(char * fname, char * MD5)
*		Checks local log DB to see if this log line has been checked 
*		before. Matches against the file name and MD5 of log line.
*		if found returns 1, if not present 0.
*
*	Args :
*		char * fname	Path and filename of log file being checked.
*		char * MD5	MD5 of log line.
*
*       Updated :
******************************************************************************/
int Check_Local_Log_UniqueEntry(char * fname, char * MD5)
{
	struct LOGEntry {
				char		fname[255];
				char		MD5[35];
				unsigned	pos;
			} logentry, flogentry, plogentry;
	
	FILE *fp;
	int found = 0;
	unsigned pos = 0;
	char * LogFile = 0;
	unsigned CacheSize = 0;
		
	LogFile = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DBFILE.LOG");
	CacheSize = Myatoll(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DBFILE.LOG.CACHESIZE"));
	
	if(LogFile && CacheSize)
	{
		fp = fopen(LogFile, "r+b");
		if(!fp)
		{
			fp = fopen(LogFile, "w+b");
			if(fp)
			{
		        	plogentry.pos = 1;
		  		fwrite(&plogentry, sizeof(struct LOGEntry), 1, fp);
		        	strncpy(logentry.fname, fname, 254);
		        	strncpy(logentry.MD5, MD5, 34);
		        	fwrite(&logentry, sizeof(struct LOGEntry), 1, fp);
		
				fclose(fp);
	       		 	return(1);
			}    
		} else {
    			rewind(fp);
			fread(&plogentry, sizeof(struct LOGEntry), 1, fp);
 			do
			{
				pos++;
				fread(&flogentry, sizeof(struct LOGEntry), 1, fp);
				if(!feof(fp))
				{
					if(!strcasecmp(flogentry.fname, fname) && !strcasecmp(flogentry.MD5, MD5))
					{
						fclose(fp);
						found = 1;
						break;
					}
				}
			}while(!feof(fp));
		
			if(found == 0)
			{
		      		plogentry.pos++;
		      		if(plogentry.pos >= CacheSize)
		      		{
		      			plogentry.pos = 1;
	      			}
	
		      		fseek(fp, (plogentry.pos * sizeof(struct LOGEntry)), SEEK_SET);
		      		strncpy(logentry.fname, fname, 254);
		      		strncpy(logentry.MD5, MD5, 34);
       	 	    		fwrite(&logentry, sizeof(struct LOGEntry), 1, fp);
       	         
         	   		rewind(fp);
         	   		fwrite(&plogentry, sizeof(struct LOGEntry), 1, fp);
         	   		fclose(fp);
         	   		return(1);
			} else {
				return(0);
			}
		}
	}
	return(0);	
}

/******************************************************************************
*       char * Check_Local_RAM(struct XMLLeaf * CheckLeaf)
*		Performs RAM check. Uses the CONFIG.CMD.RAM (free) to check
*		RAM and SWAP.
*
*	Args :
*		struct XMLLeaf * CheckLeaf	XML linked list containing 
*						check options.
*
*       Updated :
******************************************************************************/
char * Check_Local_RAM(struct XMLLeaf * CheckLeaf)
{
	char Line[255];
	char * Response = 0;
	char * tResponse = 0;
	char * Detail = 0;
	char * tDetail = 0;
	char * CMD = 0;
	unsigned RAMFREE = 0;
	unsigned SWAPFREE = 0;
	unsigned RamFree = 0;
	unsigned SwapFree = 0;
	int Status = 1;
	FILE *pp;
	char * STAT = 0;
	char * tSTAT = 0;
			
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Local_RAM]-[Enter]\n");
	
	Response = dsprintf(Response, "<CHECK><HID>%s</HID><CID>%s</CID>", 
				    ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.HID"),
				    ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.CID"));
	
	if(CheckLeaf)
	{
		RAMFREE = Myatoll(ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.RAM.RAMFREE"));
		SWAPFREE = Myatoll(ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.RAM.SWAPFREE"));
		CMD = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.CMD.RAM");
		
		MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Local_RAM]-[CMD:%s]-[RamFree:%u]-[SwapFree:%u]\n", CMD, RAMFREE, SWAPFREE);
		if(CMD)
		{
			pp = popen(CMD, "r");
			if(pp)
			{
				fgets(Line, 254, pp);
					
				fgets(Line, 254, pp);
				strtok(Line, " ");
				strtok(NULL, " ");
				strtok(NULL, " ");
				RamFree = Myatoll(strtok(NULL, " "));
				
				fgets(Line, 254, pp);
				
				fgets(Line, 254, pp);
				strtok(Line, " ");
				strtok(NULL, " ");
				strtok(NULL, " ");
				SwapFree = Myatoll(strtok(NULL, " "));

				MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Local_Ram]-[Ramfree:%u]-[Swapfree:%u]", RamFree, SwapFree);
				
				if(RAMFREE)
				{
					if(RAMFREE >= RamFree)
					{
						Status = 0;
						tDetail = dsprintf(tDetail, "[RED] Ram : %u free. (Minumum %u)\n", RamFree, RAMFREE);
					} else {
						tDetail = dsprintf(tDetail, "[GREEN] Ram : %u free. (Minumum %u)\n", RamFree, RAMFREE);
					}
					
					tSTAT = dsprintf(tSTAT, "<STAT><TYPE>CHECK</TYPE><ID>%s_%s</ID><ORDER>0</ORDER><LABEL>Ram Free</LABEL><DATAVAL>%u</DATAVAL></STAT>",
				    				ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.CID"), 
				    				ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),
				    				RamFree);
					STAT = dstrcat(STAT, tSTAT);
					tSTAT = dfree(tSTAT);
				}
				Detail = dstrcat(Detail, tDetail);
				tDetail = dfree(tDetail);
				
				if(SWAPFREE)
				{
					if(SWAPFREE >= SwapFree)
					{
						Status = 0;
						tDetail = dsprintf(tDetail, "[RED] Swap : %u free. (Minumum %u)\n", SwapFree, SWAPFREE);
					} else {
						tDetail = dsprintf(tDetail, "[GREEN] Swap : %u free. (Minumum %u)\n", SwapFree, SWAPFREE);
					}
				
					tSTAT = dsprintf(tSTAT, "<STAT><TYPE>CHECK</TYPE><ID>%s_%s</ID><ORDER>1</ORDER><LABEL>Swap Free</LABEL><DATAVAL>%u</DATAVAL></STAT>",
				    				ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.CID"), 
				    				ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),
				    				SwapFree);
					STAT = dstrcat(STAT, tSTAT);
					tSTAT = dfree(tSTAT);
				}
				Detail = dstrcat(Detail, tDetail);
				tDetail = dfree(tDetail);
			}	
			pclose(pp);					
		}
	}
	
	if(!Detail)
        {
                Detail = dstrcpy(Detail, "No Checks to perform");
        }

	if(!STAT)
	{
		STAT = dstrcpy(STAT, "");
	}
	
	Detail = CleanXMLValueString(Detail);
	
	tResponse = dsprintf(tResponse, "<STATUS>%s</STATUS><DETAIL>%s</DETAIL>%s</CHECK>", STATUS[Status], Detail, STAT);
	Response = dstrcat(Response, tResponse);
	tResponse = dfree(tResponse);
	
	STAT = dfree(STAT);
	Detail = dfree(Detail);

	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[Check_Local_RAM]-[Exit]");
		
	return(Response);
}

/******************************************************************************
*       char * Check_Local_MD5(struct XMLLeaf * RootLeaf)
*		Performs MD5 file integrity checks.
*
*	Args :
*		struct XMLLeaf * RootLeaf	XML linked list conatining
*						check options.
*
*       Updated :
******************************************************************************/
char * Check_Local_MD5(struct XMLLeaf * RootLeaf)
{
	struct MD5Record md5record;
	
	struct XMLLeaf * LocalLeaf;
	struct XMLLeaf * CheckLeaf;
	
	char * PATH = 0;
	char * RECUR = 0;
	int RecurFlag = 0;
	int Status = 1;
	char * Detail = 0;
	char * tDetail = 0;
	char * Response = 0;
	char * tResponse = 0;
	struct MD5DirTreeResult * LocalDirTreeResult = 0;
	char * STAT = 0;
	char * tSTAT = 0;
	int OptionCount = 0;
		
	Response = dsprintf(Response, "<CHECK><HID>%s</HID><CID>%s</CID>", 
				    ReturnXMLLeafValue(RootLeaf, "SERVER.RETURNCHECKS.CHECK.HID"),
				    ReturnXMLLeafValue(RootLeaf, "SERVER.RETURNCHECKS.CHECK.CID"));
				    
	LocalLeaf = RootLeaf;
	CheckLeaf = RootLeaf;
	while(LocalLeaf)
	{
		CheckLeaf = BuildSubXMLTree(LocalLeaf, ReturnXMLLeafID(LocalLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.MD5"), 1);
		
		if(CheckLeaf)
		{
			
			PATH = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.MD5.PATH");
			RECUR = ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.OPTION.MD5.RECURSIVE");
					
			if(PATH && RECUR)
			{
				if(strcasecmp(RECUR, "TRUE"))
				{
					RecurFlag = 1;
				} else {
					RecurFlag = 0;
				}
				
				strcpy(md5record.fname, PATH);
				LocalDirTreeResult = Check_Local_MD5_DirTree(md5record, RecurFlag);
				
				if(LocalDirTreeResult)
				{
					if(LocalDirTreeResult->Num)
					{
						Status = 0;
						Detail = dstrcat(Detail, LocalDirTreeResult->Result);
					}
					
					tSTAT = dsprintf(tSTAT, "<STAT><TYPE>CHECK</TYPE><ID>%s_%s</ID><ORDER>%d</ORDER><LABEL>%s</LABEL><DATAVAL>%d</DATAVAL></STAT>",
				    				ReturnXMLLeafValue(RootLeaf, "SERVER.RETURNCHECKS.CHECK.CID"),
				    				ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"),
				    				OptionCount,
				    				PATH,
				    				LocalDirTreeResult->Num);
					STAT = dstrcat(STAT, tSTAT);
					tSTAT = dfree(tSTAT);
					
					OptionCount++;
				}
				LocalDirTreeResult->Result = dfree(LocalDirTreeResult->Result);
				LocalDirTreeResult = (struct MD5DirTreeResult *) dfree((char *) LocalDirTreeResult);
			}
			LocalLeaf = FindXMLLeafByID(LocalLeaf, CheckLeaf->ID);
			CheckLeaf = DelSubXMLTree(CheckLeaf);	
		}
		
		if(LocalLeaf)
		{
			LocalLeaf = LocalLeaf->NextLeaf;
		}
	}
	
	tDetail = Check_Local_MD5_Deleted();
	if(tDetail)
	{
		Detail = dstrcat(Detail, tDetail);
		Status = 0;
	}
	tDetail = dfree(tDetail);
	
	if(!Detail)
	{
		Detail = dstrcpy(Detail, "[GREEN] No modified files.");
	}
	
	if(!STAT)
	{
		STAT = dstrcpy(STAT, "");
	}
	
	tResponse = dsprintf(tResponse, "<STATUS>%s</STATUS><DETAIL>%s</DETAIL><RRD>%s</RRD></CHECK>", STATUS[Status], Detail, STAT);
	Response = dstrcat(Response, tResponse);
	tResponse = dfree(tResponse);
	
	Detail = dfree(Detail);
	STAT = dfree(STAT);
	
	return(Response);
}

/******************************************************************************
*       MD5DirTreeResult * Check_Local_MD5_DirTree(struct MD5Record md5record, int RecurFlag)
*		Recursive function for reading in the file stats and file MD5 
*		signature. Will update local DB. 
*
*	Args :
*		struct MD5Record md5record	Struct containing parent/root
*						md5 record.
*		int RecurFlag			If true causes function to 
*						recurse down directories
*       Updated :
******************************************************************************/
struct MD5DirTreeResult * Check_Local_MD5_DirTree(struct MD5Record md5record, int RecurFlag)
{
	DIR *dp;
	struct dirent *de;
	struct stat sb;
	char * MD5 = 0;
	char * Dir = 0;
	char * Detail = 0;
	char * tDetail = 0;
	struct MD5DirTreeResult * LocalDirTreeResult = 0;
	struct MD5DirTreeResult * ReturnDirTreeResult = 0;
	
	struct MD5Record lmd5record;
	
	chdir(md5record.fname);
	dp = opendir(md5record.fname);
	if(dp)
	{
		LocalDirTreeResult = (struct MD5DirTreeResult *) drealloc((char *)LocalDirTreeResult, sizeof(struct MD5DirTreeResult));
		do
		{
			de = readdir(dp);
			if(de)
			{
				Dir = dsprintf(Dir, "%s/%s", md5record.fname, de->d_name);
				stat(Dir, &sb);
			
				if(S_ISDIR(sb.st_mode))
				{
					if(strcmp(de->d_name, ".") && strcmp(de->d_name, ".."))
					{
						if(RecurFlag)
						{
							strcpy(lmd5record.fname, Dir);
							ReturnDirTreeResult = Check_Local_MD5_DirTree(lmd5record, RecurFlag);
							if(ReturnDirTreeResult)
							{
								LocalDirTreeResult->Result = dstrcat(LocalDirTreeResult->Result, ReturnDirTreeResult->Result);
								LocalDirTreeResult->Num =+ ReturnDirTreeResult->Num;
							}
							LocalDirTreeResult->Result = dfree(LocalDirTreeResult->Result);
							ReturnDirTreeResult = (struct MD5DirTreeResult *) dfree((char *) ReturnDirTreeResult);
						}
					}
				} else {
					MD5 = MD5File(Dir);

	  				sprintf(lmd5record.fname, "%s", Dir);
	  				sprintf(lmd5record.md5, "%s", MD5);
	  				lmd5record.gid = sb.st_gid;
	  				lmd5record.uid = sb.st_uid;
	  				lmd5record.size = sb.st_size;
	  				lmd5record.ctime = sb.st_ctime;
	  				lmd5record.mtime = sb.st_mtime;
	  				lmd5record.atime = sb.st_atime;

	  				if(MD5)
	  				{
	        				if(!Check_Local_MD5_CheckDB(&lmd5record))
	        				{
	        					tDetail = dsprintf(tDetail, "--------------------------------------\nFile Name : %s \nFile size : %ld\nUID/GID : %d/%d\nMD5 : %s\n",
	        							             lmd5record.fname, lmd5record.size, lmd5record.uid, 
	        							             lmd5record.gid, lmd5record.md5);
	        					
	        					LocalDirTreeResult->Result = dstrcat(LocalDirTreeResult->Result, tDetail);
	        					LocalDirTreeResult->Num++;
	        					
	        					tDetail = dfree(tDetail);
	   						Check_Local_MD5_UpdateDB(&lmd5record);
	        				}
   					}
   					MD5 = dfree(MD5);
   				}
				Dir = dfree(Dir);
   			}
   		}while(de);
   	} else {
   		Detail = dsprintf(Detail, "[RED] Unable to open directory %s\n", md5record.fname);
   	}
   	
   	closedir(dp);
   	
   	return(LocalDirTreeResult);
}      

/******************************************************************************
*       int Check_Local_MD5_CheckDB(struct MD5Record * md5record)
*		Checks to see if md5record is different or not present in 
*		local MD5 DB. Returns 1 is found and same, 0 if different/not
*		present.
*
*	Args :
*		struct MD5Record * md5record	MD5 struct record to check.
*
*       Updated :
******************************************************************************/
int Check_Local_MD5_CheckDB(struct MD5Record * md5record)
{
	FILE *fp;
	struct MD5Record lmd5record;
	char * MD5FILE = 0;

	MD5FILE = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DBFILE.MD5");
	if(MD5FILE)
	{
		fp = fopen(MD5FILE,"r+b");
		if(fp)
		{
			do
			{
				fread(&lmd5record, sizeof(struct MD5Record), 1, fp);

				if(!strcmp(md5record->fname, lmd5record.fname) && 
				   !strcmp(md5record->md5, lmd5record.md5) && 
				   md5record->uid == lmd5record.uid && 
				   md5record->gid == lmd5record.gid &&
				   md5record->ctime == lmd5record.ctime &&
			   	   md5record->mtime == lmd5record.mtime)
				{
					fclose(fp);
					return(1);
				}
			}while(!feof(fp));
			fclose(fp);
		}
	}
	return(0);
}

/******************************************************************************
*	void Check_Local_MD5_UpdateDB(struct MD5Record *md5record)
*		Updates/Creates Local MD5 DB with passed MD5 record.
*
*	Args :
*		struct MD5Record *md5record	MD5 record to update/insert.
*
*       Updated :
******************************************************************************/
void Check_Local_MD5_UpdateDB(struct MD5Record *md5record)
{
	FILE *fp;
	int found = 0;
	struct MD5Record lmd5record;
	unsigned fpos;
	char * MD5FILE = 0;

        MD5FILE = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DBFILE.MD5");
        if(MD5FILE)
        { 
        	fp = fopen(MD5FILE,"r+b");
        	if(!fp)
        	{
        		fp = fopen(MD5FILE, "w+b");
        	}
        
	        if(fp)
	        {
	                do
	                {
	        		fpos = ftell(fp);
	                 	fread(&lmd5record, sizeof(struct MD5Record), 1, fp);
				if(lmd5record.fname)
				{
	                        	if(!strcmp(md5record->fname, lmd5record.fname))
	                        	{
						fseek(fp, fpos, SEEK_SET);
						fwrite(md5record, sizeof(struct MD5Record), 1, fp);
						found = 1;
						break;
					}
				}
	                }while(!feof(fp));
	        	
       		 	if(found == 0)
			{
				fwrite(md5record, sizeof(struct MD5Record), 1, fp);
			}

                	fclose(fp);
        	}
        }
}

/******************************************************************************
*       char * Check_Local_MD5_Deleted(void)
*		Finds Files that are present in Local MD5 DB but not present in
*		file system anymore.
*
*
*       Updated :
*		11/24/2003 - If no update it was not removing tmp file. Now 
*			     does this.
******************************************************************************/
char * Check_Local_MD5_Deleted(void)
{
	FILE *fp;
	FILE *fp2;
	char * MD5FILE = 0;
	char * TMPFILE = 0;
	char * TMPCMD = 0;
	struct MD5Record lmd5record;
	char * Response = 0;
	char * tResponse = 0;
	int Update = 0;
			
	MD5FILE = ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DBFILE.MD5");
	if(MD5FILE)
	{
		fp = fopen(MD5FILE, "r+b");
		if(fp)
		{
			TMPFILE = dsprintf(TMPFILE, "/tmp/MD5tmp.db.%d", getpid());
			fp2 = fopen(TMPFILE, "w+b");
			if(fp2)
			{
				do
				{
					fread(&lmd5record, sizeof(struct MD5Record), 1, fp);
					if(lmd5record.fname)
					{
						if(!FilePresent(lmd5record.fname))
						{
							Update = 1;
						
							tResponse = dsprintf(tResponse, "--------------------------------------\nMissing file: %s\n", lmd5record.fname);
							Response = dstrcat(Response, tResponse);
							tResponse = dfree(tResponse);
						} else {
							fwrite(&lmd5record, sizeof(struct MD5Record), 1, fp2);
						}
					}
				}while(!feof(fp));
				fflush(NULL);
				
				fclose(fp2);
				fclose(fp);
				
				if(Update)
				{
					TMPCMD = dsprintf(TMPCMD, "mv %s %s", TMPFILE, MD5FILE);
					system(TMPCMD);
					TMPCMD = dfree(TMPCMD);				
				} else {
					unlink(TMPFILE);
				}
			}
		}
	}
	return(Response);
}
