/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASc
*       Filename : processchecks.c
*               Function to process the return check queue given back 
*		from the RETURNCHECKS query.
*
*       Updated :
*               11/07/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <fcntl.h>

#include "includes.h"
#include "extern.h"
#include "../Common/Commoninclude.h"

/******************************************************************************
*       char * ProcessChecks(struct XMLLeaf * RootXMLTree)
*		Main entry function when processesing check queue. Fires correct
*		check function based on CheckType.
*
*	Args :
*		struct XMLLeaf * RootXMLTree	XML linked list containing
*						check queue.
*
*       Updated :
******************************************************************************/
char * ProcessChecks(struct XMLLeaf * RootXMLTree)
{
	char * Response = 0;
	char * tResponse = 0;
	struct XMLLeaf * LocalLeaf = 0;
	struct XMLLeaf * CheckLeaf = 0;
	int CheckType = 0;
	int Pipe[2];
	int FreeChildrenCount = 0;
	unsigned CHILDREN[255];
	unsigned BIRTH[255];
	pid_t child;
	
	bzero(&CHILDREN, sizeof(CHILDREN));
	bzero(&BIRTH, sizeof(BIRTH));
				
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ProcessChecks]-[Enter]-[Checks to process:%u]", CountXMLLeafByTag(RootXMLTree, "SERVER.RETURNCHECKS.CHECK"));
	
	pipe(Pipe);
	
	Response = dsprintf(Response, "<CLIENT><VER>%s</VER><HID>%s</HID><TYPE>CHECKUPDATE</TYPE><CHECKUPDATE>", MIDASVER, ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"));

	LocalLeaf = RootXMLTree;
	CheckLeaf = RootXMLTree;
	while(LocalLeaf)
	{
		CheckLeaf = BuildSubXMLTree(LocalLeaf, ReturnXMLLeafID(LocalLeaf, "SERVER.RETURNCHECKS.CHECK"), 1);
		if(CheckLeaf)
		{
			CheckType = Myatoi(ReturnXMLLeafValue(CheckLeaf, "SERVER.RETURNCHECKS.CHECK.TYPE"));
			if(CheckType)
			{
				do
				{
					FreeChildrenCount = FreeChild(CHILDREN, BIRTH);
					if(FreeChildrenCount != -1)
					{
						child = fork();
						if(child == 0)
						{
							MyDebug(__LINE__, __FILE__, LOG_INFO, "[ProcessChecks]-[CheckType:%d]", CheckType);
							switch(CheckType)
							{
								case 1:	// TCP
									tResponse = Check_Network_TCP(CheckLeaf);
									break;
								case 2: // UDP
									tResponse = Check_Network_UDP(CheckLeaf);
									break;
								case 3: // ICMP
									tResponse = Check_Network_ICMP(CheckLeaf);
									break;
								case 4: // Local Disk
									tResponse = Check_Local_Disk(CheckLeaf);
									break;
								case 5: // Local CPU
									tResponse = Check_Local_CPU(CheckLeaf);
									break;
								case 6: // Local Proc
									tResponse = Check_Local_Proc(CheckLeaf);
									break;
								case 7: // Local Log
									tResponse = Check_Local_Logs(CheckLeaf);
									break;
								case 8: // SNMP
									tResponse = Check_Network_SNMP(CheckLeaf);
									break;
								case 9: // External MIDAS
									tResponse = Check_External_MIDAS(CheckLeaf);
									break;
								case 10: // External NETSAINT
									tResponse = Check_External_NetSaint(CheckLeaf);
									break;
								case 11: // Local RAM
									tResponse = Check_Local_RAM(CheckLeaf);
									break;
								case 12: // Local MD5
									tResponse = Check_Local_MD5(CheckLeaf);
									break;
							}
							
							write(Pipe[1], tResponse, strlen(tResponse));
							tResponse = dfree(tResponse);
							close(Pipe[0]);
							close(Pipe[1]);

							exit(0);
						} else {
							AddChild(child, CHILDREN, BIRTH);
						}
					} else {
						usleep(100);
						continue;
					}
				}while(FreeChildrenCount == -1);
				
				tResponse = ReadPipe(Pipe[0]);
				Response = dstrcat(Response, tResponse);
				tResponse = dfree(tResponse);
			}
			LocalLeaf = FindXMLLeafByID(LocalLeaf, CheckLeaf->ID);
			CheckLeaf = DelSubXMLTree(CheckLeaf);
		}
		
		if(LocalLeaf)	
		{
			LocalLeaf = LocalLeaf->NextLeaf;
		}
	}

   	do
        {
		tResponse = ReadPipe(Pipe[0]);
		if(tResponse)
		{	
			Response = dstrcat(Response, tResponse);
			sleep(1);
		}
		tResponse = dfree(tResponse);
   	}while(NoChild(CHILDREN, BIRTH));	
	
	close(Pipe[0]);
	close(Pipe[1]);
	
	Response = dstrcat(Response, "</CHECKUPDATE></CLIENT>");
	
	return(Response);
}

/******************************************************************************
*       char * ReadPipe(int Pipes)
*		Reads the piped output of spawned child process. Used by parent
*		process to read resulting response of childs check.
*
*	Args :
*		int Pipes	Pipe handle of child processe's stdout.
*
*       Updated :
******************************************************************************/
char * ReadPipe(int Pipes)
{
	char * OutStr = 0;
	char ReadBuff[1024];
	int len = 0;		
	struct timeval tv;
	fd_set sel_P;
	int status;
	
	FD_ZERO(&sel_P);
	FD_SET(Pipes, &sel_P);
	do
	{
		tv.tv_sec = 0;
		tv.tv_usec = 10;
		status =  select(Pipes+1, &sel_P, NULL, NULL, &tv);
		if(status)
		{
			len = read(Pipes, ReadBuff, 1023);
			if(len > 0)
			{
				ReadBuff[len]=0;
				OutStr = dstrcat(OutStr, ReadBuff);
			}
		}
		usleep(100);
	}while(status && status != -1 && len != -1);
	
	return(OutStr);

}
