/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASd
*       Filename : processclient.c
* 		Functions needed to process client requests.
*
*       Updated :
*               11/07/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <time.h>

#include "../Common/Commoninclude.h"

#include "includes.h"
#include "extern.h"

/******************************************************************************
*       char * ProcessClientRequest(struct XMLLeaf * ClientXMLTree)
*		Main entry function for processing client requests. Fires off
*		specific function based on value of CHECK.TYPE tag in client 
*		request.
*
*	Args :
*		struct XMLLeaf * ClientXMLTree		XML linked list cont-
*							aining check request
*							options.
*
*       Updated :
*		11/20/2003 - Sanity check added. Valid HID required before 
*			     accepting client query.
******************************************************************************/
char * ProcessClientRequest(struct XMLLeaf * ClientXMLTree)
{
	struct XMLLeaf * LocalLeaf;
	char * Response = 0;
	char * tResponse = 0;
	char * HID;
	
	Response = dsprintf(Response, "<SERVER><VER>%s</VER><HID>%s</HID>", MIDASVER, ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"));
	
	LocalLeaf = ClientXMLTree;
	if(LocalLeaf)
	{
		HID = ReturnXMLLeafValue(LocalLeaf, "CLIENT.HID");
		if(ValidHID(HID))
		{
			while(LocalLeaf)
			{
				LocalLeaf = FindXMLLeafByTag(LocalLeaf, "CLIENT.TYPE");
				if(LocalLeaf)
				{
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[ProcessClientRequest]-[HID:%s]-[Type:%s]", HID, LocalLeaf->Value);
					
					if(!strcasecmp(LocalLeaf->Value, "RETURNCHECKS"))
					{
						tResponse = ReturnClientChecks(HID);
					}
					
					if(!strcasecmp(LocalLeaf->Value, "CHECKUPDATE"))
					{
						tResponse = UpdateCheckStatus(LocalLeaf, HID);
					}
					
					if(!strcasecmp(LocalLeaf->Value, "SNORTCONFIG"))
					{
						tResponse = ReturnSnortConfig(HID);
					}
					
					if(!strcasecmp(LocalLeaf->Value, "SNORTALERT"))
					{
						tResponse = UpdateSnortAlert(LocalLeaf, HID);
					}
					
					if(!strcasecmp(LocalLeaf->Value, "HIDQUERY"))
					{
						tResponse = ReturnHIDQuery(LocalLeaf);
					}
					
					if(!strcasecmp(LocalLeaf->Value, "NETSTAT"))
					{
						tResponse = UpdateNetStats(LocalLeaf, HID);
					}
					
					if(!strcasecmp(LocalLeaf->Value, "RETURNNETMON"))
					{
						tResponse = ReturnNetMon(HID);
					}
					
					Response = dstrcat(Response, tResponse);
					tResponse = dfree(tResponse);
					
					LocalLeaf = LocalLeaf->NextLeaf;
				}			
			}
			
		}	
	
		LocalLeaf = ClientXMLTree;
		if(FindXMLLeafByTag(LocalLeaf,  "CLIENT.STAT"))
		{
			ProcessClientStats(LocalLeaf, "CLIENT", HID);
		}
	} else {
		Response = dstrcat(Response, "<ERROR>INVALID_HID</ERROR>");
	}		
	
	Response = dstrcat(Response, "</SERVER>");
	return(Response);
}

/******************************************************************************
*	int ValidHID(char * HID)
*		Returns 1 if HID is valid (present) or 0 if not.
*
*	Args :
*		char * HID	Host ID to check.
*
*	Updated :
*
******************************************************************************/
int ValidHID(char * HID)
{
	struct DBR * dbr = 0;	
        char * query = 0;
        int Status = 0;
        
        query = dsprintf(query, "SELECT HID FROM MIDAS_Hosts WHERE HID='%s'", HID);
        dbr = dbQuery(query, SQLCFG);
        if(dbr)
        {
        	Status = dbr->num;
        }
        dbr = dbFree(dbr);
        
        return(Status);
}

/******************************************************************************
*       char * ReturnClientChecks(char * HID)
*		Response for RETURNCHECKS query. This will build the list of
*		checks for the requesting MIDASc to perform.
*
*	Args :
*		char * HID	The Host ID of the requesting MIDASc.
*
*       Updated :
******************************************************************************/
char * ReturnClientChecks(char * HID)
{
	struct DBR * dbr = 0;
        MYSQL_ROW myrow;
        char * query = 0;
        int i;
        char * Response = 0;
        char * Segment = 0;

	char SELECT[] = "SELECT MIDAS_Host_Checks.HID, MIDAS_Host_Checks.CID, MIDAS_Hosts.HostName, MIDAS_Hosts.IPAddr, MIDAS_Checks.Type, "
			"MIDAS_Host_Checks.Options, MIDAS_Checks.Port, MIDAS_Checks.ConnectOnly "
	                "FROM MIDAS_Host_Checks "
	                "LEFT JOIN MIDAS_Checks ON (MIDAS_Host_Checks.CID = MIDAS_Checks.CID) "
	                "LEFT JOIN MIDAS_Hosts ON (MIDAS_Hosts.HID = MIDAS_Host_Checks.HID) "
	        	"WHERE MIDAS_Host_Checks.TID='%s' AND MIDAS_Host_Checks.Enabled='1'";
		
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturnClientChecks]-[HID:%s]", HID);
	
        query = dsprintf(query, SELECT, HID);
        dbr = dbQuery(query, SQLCFG);
	query = dfree(query);
	
	if(dbr)
	{
		if(dbr->num)
		{
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturnClientChecks]-[Number of Checks:%u]", dbr->num);
			Response = dstrcat(Response, "<RETURNCHECKS>");
        		for(i=0;i<dbr->num;i++)
        		{
        		        myrow = dbResult(dbr);
		
				Segment = dsprintf(Segment, "<CHECK><HID>%s</HID><CID>%s</CID><HOSTNAME>%s</HOSTNAME><IP>%s</IP><TYPE>%s</TYPE><OPTION>%s</OPTION><PORT>%s</PORT><CONNECTONLY>%s</CONNECTONLY></CHECK>",
							    myrow[0], myrow[1], myrow[2], myrow[3], myrow[4], myrow[5], myrow[6], myrow[7]);
				Response = dstrcat(Response, Segment);
				Segment = dfree(Segment);
		        }
			Response = dstrcat(Response, "</RETURNCHECKS>");
		}
	}
	dbr = dbFree(dbr);
	return(Response);
}

/******************************************************************************
*       char * UpdateCheckStatus(struct XMLLeaf * ClientXMLTree, char * TID)
*		Updats the status of checks based on the results returned by
*		a MIDASc client. This is used for CHECKUPDATE request.
*
*	Args :
*		struct XMLLeaf * ClientXMLTree		XML linked list cont-
*							aining check statuses.
		char * TID				The MIDASc's Host ID.
*
*       Updated :
*		11/18/03 - CurFail now resets when an alert is issued.
*		11/20/03 - Fixed Update record. Now retains FirstCheck value.
******************************************************************************/
char * UpdateCheckStatus(struct XMLLeaf * ClientXMLTree, char * TID)
{
	struct XMLLeaf * LocalLeaf = 0;
	struct XMLLeaf * CheckLeaf = 0;
	struct DBR * dbr = 0;
        MYSQL_ROW myrow;
	char * query = 0;
	char * Response = 0;
	char * HID = 0;
	char * CID = 0;
	char * STATUS = 0;
	char * DETAIL = 0;
	unsigned CheckCount = 0;
	int AlertOnRed = 0;	
	
	LocalLeaf = ClientXMLTree;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[TID:%s]", TID);
	
	LocalLeaf = ClientXMLTree;
		
	while(LocalLeaf)
	{
		CheckLeaf = BuildSubXMLTree(LocalLeaf, ReturnXMLLeafID(LocalLeaf, "CLIENT.CHECKUPDATE.CHECK"), 1);
		if(CheckLeaf)
		{
			CheckCount++;
			HID = ReturnXMLLeafValue(CheckLeaf, "CLIENT.CHECKUPDATE.CHECK.HID");
			CID = ReturnXMLLeafValue(CheckLeaf, "CLIENT.CHECKUPDATE.CHECK.CID");
			STATUS = ReturnXMLLeafValue(CheckLeaf, "CLIENT.CHECKUPDATE.CHECK.STATUS");
			DETAIL = ReturnXMLLeafValue(CheckLeaf, "CLIENT.CHECKUPDATE.CHECK.DETAIL");
			
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[HID:%s]-[CID:%s]-[STATUS:%s]", 
							       HID, CID, STATUS);
			
			query = dsprintf(query, "SELECT Status FROM MIDAS_Check_Current WHERE TID='%s' AND HID='%s' AND CID='%s'",
							TID, HID, CID);
			dbr = dbQuery(query, SQLCFG);
			query = dfree(query);
			if(dbr)
			{
				if(dbr->num)
				{
					myrow = dbResult(dbr);
					
					AlertOnRed = 0;
					if(!strcasecmp(STATUS, "RED"))
					{
						query = dsprintf(query, "UPDATE MIDAS_Host_Checks SET CurFail=CurFail+1, TotFail=TotFail+1 WHERE TID='%s' AND HID='%s' AND CID='%s'",
									TID, HID, CID);
						dbr = dbQuery(query, SQLCFG);
						query = dfree(query);
						dbr = dbFree(dbr);
						
						query = dsprintf(query, "SELECT FROM MIDAS_Checks WHERE CID='%s' AND AlertOnRed=1", CID);
						dbr = dbQuery(query, SQLCFG);
						if(dbr)
						{
							if(dbr->num)
							{
								AlertOnRed = 1;
							}
						}
						dbr = dbFree(dbr);
					}
					
					if(strcasecmp(myrow[0], STATUS) || AlertOnRed == 1)
					{
						MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[Status Change]-[%s -> %s]-[alertOnRed:%d]", myrow[0], STATUS, AlertOnRed);
						dbr = dbFree(dbr);
						
						query = dsprintf(query, "UPDATE MIDAS_Check_Current SET Status='%s', Response='%s', FirstCheck=NOW(), LastCheck=NOW() WHERE TID='%s' AND HID='%s' AND CID='%s'",
										STATUS, DETAIL, TID, HID, CID);
						dbr = dbQuery(query, SQLCFG);
						query = dfree(query);
						dbr = dbFree(dbr);
						
						if(strcasecmp(myrow[0], "BLUE") || AlertOnRed == 1)
						{
							query = dsprintf(query, "INSERT INTO MIDAS_Check_Events (TID, HID, CID, Response, Status, TimeStamp) VALUES ('%s', '%s', '%s', '%s', '%s', NOW())",
										     TID, HID, CID, DETAIL, STATUS);
							dbr = dbQuery(query, SQLCFG);
							query = dfree(query);
							dbr = dbFree(dbr);
							
							query = dsprintf(query, "UPDATE MIDAS_Check_Hosts SET CurFail=0 WHERE TID='%s' AND HID='%s' AND CID='%s'", TID, HID, CID);
							dbr = dbQuery(query, SQLCFG);
							query = dfree(query);
							dbr = dbFree(dbr);
							
							Alert_Process(CheckLeaf, TID);
						}
					} else {
						MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[No Status Change]-[%s = %s]", myrow[0], STATUS);
						dbr = dbFree(dbr);

						query = dsprintf(query, "UPDATE MIDAS_Check_Current SET Status='%s', Response='%s', LastCheck=NOW(), FirstCheck=FirstCheck WHERE TID='%s' AND HID='%s' AND CID='%s'",
										STATUS, DETAIL, TID, HID, CID);
						dbr = dbQuery(query, SQLCFG);
						query = dfree(query);
						dbr = dbFree(dbr);
					}
					
				} else {
					MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[New Event]");
					dbr = dbFree(dbr);
					query = dsprintf(query, "INSERT INTO MIDAS_Check_Current (TID, HID, CID, Response, Status, LastCheck, FirstCheck) VALUES ('%s', '%s', '%s', '%s', '%s', NOW(), NOW())",
									     TID, HID, CID, DETAIL, STATUS);
					dbr = dbQuery(query, SQLCFG);
					query = dfree(query);
					dbr = dbFree(dbr);
					
					query = dsprintf(query, "INSERT INTO MIDAS_Check_Events (TID, HID, CID, Response, Status, TimeStamp) VALUES ('%s', '%s', '%s', '%s', '%s', NOW())",
										     TID, HID, CID, DETAIL, STATUS);
					dbr = dbQuery(query, SQLCFG);
					query = dfree(query);
					dbr = dbFree(dbr);
				}
				
				if(FindXMLLeafByTag(CheckLeaf, "CLIENT.CHECKUPDATE.CHECK.STAT"))
				{
					ProcessClientStats(CheckLeaf, "CLIENT.CHECKUPDATE.CHECK", HID);
				}
			} 
			
			LocalLeaf = FindXMLLeafByID(LocalLeaf, CheckLeaf->ID);
			CheckLeaf = DelSubXMLTree(CheckLeaf);
		}
		
		if(LocalLeaf)
		{
			LocalLeaf = LocalLeaf->NextLeaf;
		}
		
	}
	Response = dstrcpy(Response, "<TYPE>CHECKUPDATE</TYPE><CHECKUPDATE>FINISHED</CHECKUPDATE>");
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateCheckStatus]-[Checks Updated:%u]-[Exiting]", CheckCount);
	return(Response);	
}

/******************************************************************************
*       char * ReturnSnortConfig(char * HID)
*		Builds the config response for MIDASs when it performs a 
*		SNORTCONFIG query.
*
*	Args :
*		char * HID	The Host id of the requesting MIDASs client.
*
*       Updated :
******************************************************************************/
char * ReturnSnortConfig(char * HID)
{
	struct DBR * dbr = 0;
        MYSQL_ROW myrow;
	char * query = 0;
	char * Response = 0;
	char * SnortConfig = 0;
	char * B64SnortConfig = 0;
	
	unsigned i = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturnSnortConfig]-[HID:%s]", HID);
	
	query = dsprintf(query, "SELECT Config FROM MIDAS_SNORT_Conf WHERE HID='%s'", HID);
	dbr = dbQuery(query, SQLCFG);
	query = dfree(query);
	
	if(dbr)
	{
		myrow = dbResult(dbr);
		SnortConfig = dstrcat(SnortConfig, myrow[0]);
	}
	dbr = dbFree(dbr);
	
	query = dsprintf(query, "SELECT Ruleset FROM MIDAS_SNORT_Rules WHERE HID='%s'", HID);
	dbr = dbQuery(query, SQLCFG);
	query = dfree(query);
	
	if(dbr)
	{
		for(i=0;i<dbr->num;i++)
		{
			myrow = dbResult(dbr);
			SnortConfig = dstrcat(SnortConfig, myrow[0]);
		}
		
		query = dsprintf(query, "UPDATE MIDAS_Snort_Rules SET ClientNSynch='1', TimeStamp=NOW() WHERE HID='%s'", HID);
		dbr = dbQuery(query, SQLCFG);
		query = dfree(query);
		dbr = dbFree(dbr);
	}
	B64SnortConfig = Base64Enc(SnortConfig, dstrlen(SnortConfig));
	SnortConfig = dfree(SnortConfig);
	SnortConfig = B64SnortConfig;

	Response = dsprintf(Response, "<TYPE>SNORTCONFIG</TYPE><SNORTCONFIG><CONFIG>%s</CONFIG></SNORTCONFIG>", SnortConfig);
	SnortConfig = dfree(SnortConfig);
	
	return(Response);
}

/******************************************************************************
*       char * UpdateSnortAlert(struct XMLLeaf * ClientXMLTree, char * HID)
*		Inserts IDS event record into MIDAS_IDS_Events. This is 
*		triggered on SNORTALERT MIDASs queries.
*
*	Args :
*		struct XMLLeaf * ClientXMLTree		XML Linked list cont-
*							aining the MIDASs 
*							query information.
*		char * HID				The Host ID of the 
*							MIDASs client.
*
*       Updated :
*		11/20/2003 - Sanity Check. Now only accepts Alerts from HID's
*			     that have isSniffer toggled on.
******************************************************************************/
char * UpdateSnortAlert(struct XMLLeaf * ClientXMLTree, char * HID)
{
	struct DBR * dbr = 0;
	char * query = 0;
	char * Response = 0;
	char * SRCMAC = 0;
	char * DSTMAC = 0;
	char * SRCIP = 0;
	char * SRCPORT = 0;
	char * DSTIP = 0;
	char * DSTPORT = 0;
	char * PROTOCOL = 0;
	char * SEQUENCE = 0;
	char * PAYLOAD = 0;
	char * ALERTDESC = 0;
	struct XMLLeaf * LocalLeaf;
	struct XMLLeaf * AlertLeaf;
	int Status = 0;
		
	LocalLeaf = ClientXMLTree;

	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[UpdateSnortAlert]-[Enter]");

	if(SnortIsSniffer(HID))
	{
		while(LocalLeaf)
		{
			AlertLeaf = BuildSubXMLTree(LocalLeaf, ReturnXMLLeafID(LocalLeaf, "CLIENT.SNORTALERT.SNORT"), 1);
			if(AlertLeaf)
			{
				SRCMAC = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.SRCMAC");
				DSTMAC = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.DSTMAC");
				SRCIP = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.SRCIP");
				SRCPORT = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.SRCPORT");
				DSTIP = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.DSTIP");
				DSTPORT = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.DSTPORT");
				PROTOCOL = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.PROTOCOL");
				SEQUENCE = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.SEQUENCE");
				PAYLOAD = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.PAYLOAD");
				ALERTDESC = ReturnXMLLeafValue(AlertLeaf, "CLIENT.SNORTALERT.SNORT.ALERTDESC");
			
				if(SRCMAC && DSTMAC && SRCIP && SRCPORT && DSTIP && DSTPORT && PROTOCOL && SEQUENCE)
				{
					query = dsprintf(query, "INSERT INTO MIDAS_IDS_Events (HID, SMAC, SIP, SPort, DMAC, DIP, DPort, Protocol, Descr, Payload, TimeStamp, Seq) VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s' , '%s', '%s', '%s', NOW(), '%s')",
										      HID, SRCMAC, SRCIP, SRCPORT, DSTMAC, DSTIP, DSTPORT, PROTOCOL, ALERTDESC, PAYLOAD, SEQUENCE);
					dbr = dbQuery(query, SQLCFG);
					query = dfree(query);
					dbr = dbFree(dbr);
				}
				LocalLeaf = FindXMLLeafByID(LocalLeaf, AlertLeaf->ID);
				AlertLeaf = DelSubXMLTree(AlertLeaf);
		
				if(FindXMLLeafByTag(AlertLeaf, "CLIENT.SNORTALERT.SNORT.STAT"))
				{
					ProcessClientStats(AlertLeaf, "CLIENT.SNORTALERT.SNORT", HID);
				}
			}
			
			if(LocalLeaf)
			{
				LocalLeaf = LocalLeaf->NextLeaf;
			}
		}
		Status = 1;
	}
	
	if(Status)
	{
		Response = dstrcpy(Response, "<TYPE>SNORTALERT</TYPE><SNORTALERT>UPDATED</SNORTALERT>");
	} else {
		Response = dstrcpy(Response, "<TYPE>SNORTALERT</TYPE><SNORTALERT>FAILED</SNORTALERT>");
	}
	
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[UpdateSnortAlert]-[Exit]");
	return(Response);
}

/******************************************************************************
*	int SnortIsSniffer(char * HID)
*		Returns 1 if HID has IsSniffer Toggled on, 0 if off.
*
*	Args :
*		char * HID	Host ID to check.
*
*	Updated :
*******************************************************************************/
int SnortIsSniffer(char * HID)
{
	struct DBR * dbr = 0;
	char * query = 0;
	int Status = 0;
		
	query = dsprintf(query, "SELECT HID FROM MIDAS_Hosts WHERE HID='%s' AND IsSniffer=1", HID);
	dbr = dbQuery(query, SQLCFG);
	if(dbr)
	{
		Status = dbr->num;
	}
	dbr = dbFree(dbr);
	
	return(Status);
}

/******************************************************************************
*       char * ReturnHIDQuery(struct XMLLeaf * ClientLeaf)
*		Builds the response for HIDQUERY. This will take a host name  
*		and return the Host ID (HID) associated to it.
*
*	Args :
*		struct XMLLeaf * ClientLeaf		XML linked list cont-
*							aining the query options.
*
*       Updated :
******************************************************************************/
char * ReturnHIDQuery(struct XMLLeaf * ClientLeaf)
{
	char * Response = 0;
	MYSQL_ROW myrow;
	char * query = 0;
	struct DBR * dbr = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[ReturnHIDQuery]-[Enter]");
	
	if(ClientLeaf)
	{
		MyDebug(__LINE__, __FILE__, LOG_CHECK, "[ReturnHIDQuery]-[Host:%s]", 
							ReturnXMLLeafValue(ClientLeaf, "CLIENT.HOSTNAME"));
							
		Response = dstrcpy(Response, "<TYPE>HIDQUERY</TYPE><HIDQUERY><HID>");
		
		query = dsprintf(query, "SELECT HID FROM MIDAS_Hosts WHERE HostName='%s'",
					ReturnXMLLeafValue(ClientLeaf, "CLIENT.HOSTNAME"));
		dbr = dbQuery(query, SQLCFG);
		query = dfree(query);
		
		if(dbr)
		{
			myrow = dbResult(dbr);
			Response = dstrcat(Response, myrow[0]);
		}
		dbr = dbFree(dbr);
		
		Response = dstrcat(Response, "</HID></HIDQUERY>");
	}
	
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[ReturnHIDQuery]-[Exit]");
	
	return(Response);
}

/******************************************************************************
*       void ProcessClientStats(struct XMLLeaf * ClientLeaf, char * SegmentName, char * HID)
*		Process the client STAT responses. Update RRD_* tables with
*		statistical information provided by the various client apps.
*
*	Args :
*		struct XMLLeaf * ClientLeaf	XML linked list containing the 
*						query options.
*		char * SegmentName		The unique Tag segment used to 
*						build the correct tag names.
*		char * HID			The Host ID of the client 
*						performing the request.
*
*       Updated :
******************************************************************************/
void ProcessClientStats(struct XMLLeaf * ClientLeaf, char * SegmentName, char * HID)
{
	struct XMLLeaf * LocalLeaf = 0;
	char * XMLStat = 0;
	char * XMLType = 0;
	char * query = 0;
	struct DBR * dbr = 0;
	char * XMLID = 0;
	char * XMLDataVal = 0;
	char * XMLOrder = 0;
	char * XMLLabel = 0;
	int NewDSNeeded = 0;
	unsigned TimeStamp = 0;
	struct RRDCreateDS * CreateDS = 0;
	struct RRDCreateArch * CreateArchMonthly = 0;
	struct RRDCreateArch * CreateArchYearly = 0;
	MYSQL_ROW myrow;		
	int DSID = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[ProcessClientStats]-[Enter]");
	
	LocalLeaf = ClientLeaf;
	if(LocalLeaf)
	{
		XMLStat = dsprintf(XMLStat, "%s.STAT", SegmentName);	
		XMLType = dsprintf(XMLType, "%s.STAT.TYPE", SegmentName);
		XMLID = dsprintf(XMLID, "%s.STAT.ID", SegmentName);
		XMLDataVal = dsprintf(XMLDataVal, "%s.STAT.DATAVAL", SegmentName);
		XMLOrder = dsprintf(XMLOrder, "%s.STAT.ORDER", SegmentName);
		XMLLabel = dsprintf(XMLLabel, "%s.STAT.LABEL", SegmentName);

		while(LocalLeaf)
		{
			NewDSNeeded = 0;
			
			LocalLeaf = FindXMLLeafByTag(LocalLeaf, XMLStat);
			if(LocalLeaf)
			{
				query = dsprintf(query, "SELECT DSID FROM RRD_MIDAS_ID WHERE Type='%s' AND ID1='%s' AND ID2='%s' AND ID3='%s'",
							ReturnXMLLeafValue(LocalLeaf, XMLType),
							HID,
							ReturnXMLLeafValue(LocalLeaf, XMLID),
							ReturnXMLLeafValue(LocalLeaf, XMLOrder));
				dbr = dbQuery(query, SQLCFG);
				query = dfree(query);
				
				if(dbr)
				{
					if(dbr->num)
					{
						myrow = dbResult(dbr);
						DSID = Myatoll(myrow[0]);
					} else {
						NewDSNeeded = 1;
					}
				} else {
					NewDSNeeded = 1;
				}
				dbr = dbFree(dbr);
				
				if(NewDSNeeded)
				{
					MyDebug(__LINE__, __FILE__, LOG_CHECK, "[ProcessClientStats]-[Creating New DataSource]");
					
					CreateDS = (struct RRDCreateDS *) drealloc((char *) CreateDS, sizeof(struct RRDCreateDS));
					CreateArchMonthly = (struct RRDCreateArch *) drealloc((char *) CreateArchMonthly, sizeof(struct RRDCreateArch));
					CreateArchYearly = (struct RRDCreateArch *) drealloc((char *) CreateArchYearly, sizeof(struct RRDCreateArch));
					
					CreateDS->Title = dsprintf(CreateDS->Title, ReturnXMLLeafValue(LocalLeaf, XMLLabel));
					CreateDS->Sequences = 288;
					CreateDS->Interval = 300;
					CreateDS->Archive = CreateArchMonthly;
				
					CreateArchMonthly->Sequences = 122;
					CreateArchMonthly->Interval = 21600;
					CreateArchMonthly->NextArch = CreateArchYearly;
				
					CreateArchYearly->Sequences = 365;
					CreateArchYearly->Interval = 86400;
					
					DSID = RRD_CreateDataSource(CreateDS);
					
					CreateDS->Title = dfree(CreateDS->Title);
					CreateDS = (struct RRDCreateDS *) dfree((char *) CreateDS);
					CreateArchMonthly = (struct RRDCreateArch *) dfree((char *) CreateArchMonthly);
					CreateArchYearly = (struct RRDCreateArch *) dfree((char *) CreateArchYearly);

					query = dsprintf(query, "INSERT INTO RRD_MIDAS_ID (DSID, Type, ID1, ID2, ID3) VALUES ('%d', '%s', '%s', '%s', '%s')",
								DSID, 
								ReturnXMLLeafValue(LocalLeaf, XMLType),
								HID,
								ReturnXMLLeafValue(LocalLeaf, XMLID),
								ReturnXMLLeafValue(LocalLeaf, XMLOrder));
					dbr = dbQuery(query, SQLCFG);
					query = dfree(query);
					dbr = dbFree(dbr);
					
					NewDSNeeded = 0;
				}	
		
				TimeStamp = time(NULL);
				
				RRD_UpdateDSValue(DSID, TimeStamp, Myatof(ReturnXMLLeafValue(LocalLeaf, XMLDataVal)));
			
				LocalLeaf = LocalLeaf->NextLeaf;
			}
		}
		XMLStat = dfree(XMLStat);
		XMLType = dfree(XMLType);
		XMLID = dfree(XMLID);
		XMLDataVal = dfree(XMLDataVal);
		XMLOrder = dfree(XMLOrder);
		XMLLabel = dfree(XMLLabel);
	}
	
	MyDebug(__LINE__, __FILE__, LOG_CHECK, "[ProcessClientStats]-[Exit]");
}

/******************************************************************************
*       char * UpdateNetStats(struct XMLLeaf * LocalLeaf, char * HID)
*		Simple front end to ProcessClientStats for MIDASn.
*
*	Args :
*		struct XMLLeaf * LocalLeaf	XML linked list containing the 
*						query options.
*		char * HID			The Host ID of the MIDASn client.
*
*       Updated :
******************************************************************************/
char * UpdateNetStats(struct XMLLeaf * LocalLeaf, char * HID)
{
	char * Response = 0;
	
	ProcessClientStats(LocalLeaf, "CLIENT.NETSTAT", HID);
	
	Response = dstrcpy(Response, "<NETSTAT>COMPLETE</NETSTAT>");
	
	return(Response);
}

/******************************************************************************
*       char * ReturnNetMon(char * HID)
*		Builds the list of Ports for MIDASn to monitor. 
*
*	Args :
*		char * HID	The Host ID of the MIDASn client.
*
*       Updated :
******************************************************************************/
char * ReturnNetMon(char * HID)
{
	char * Response = 0;
	MYSQL_ROW myrow;
	char * query = 0;
	struct DBR * dbr = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturnNetMon]-[Enter]");
	
	if(HID)
	{
		Response = dstrcpy(Response, "<TYPE>NETMON</TYPE><NETMON>");
		
		query = dsprintf(query, "SELECT Options FROM MIDAS_NetMon_Ports WHERE HID='%s'", HID);
		dbr = dbQuery(query, SQLCFG);
		
		if(dbr)
		{
			myrow = dbResult(dbr);
			Response = dstrcat(Response, myrow[0]);
		}
		
		Response = dstrcat(Response, "</NETMON>");
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[ReturnNetMon]-[Exit]");
	
	return(Response);
}
