/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASd
*       Filename : server.c
* 		Function required for the 'Server' portions of MIDASd
*
*       Updated :
*               11/07/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <mysql/mysql.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>

#include "../Common/Commoninclude.h"
#include "includes.h"
#include "extern.h"

/******************************************************************************
*       int StartServer(void)
*		Starts up the server and waits for client connects.
*
*
*       Updated :
******************************************************************************/
int StartServer(void)
{
	int sockfd, clientsockfd, len;
	struct sockaddr_in serv_addr, cli_addr;
	char *tCIP = 0;
	int FreeChildrenCount;
	unsigned CHILDREN[255];
	unsigned BIRTH[255];
	int on = 1;
	pid_t child;
	char * SendBuff = 0;
	struct XMLLeaf * ClientXMLRequest;
	char * ClientString = 0;
	char * ClientResponse = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[StartServer]-[Starting]");
	
	bzero(&CHILDREN, sizeof(CHILDREN));
	bzero(&BIRTH, sizeof(BIRTH));
	
	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof on);
	
	bzero((char *) &serv_addr, sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	serv_addr.sin_port = htons(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.Port")));
		
	if(bind(sockfd,(struct sockaddr *)&serv_addr,sizeof(serv_addr)) == -1)
	{
		MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SockServer]-[Unable to open Port for listening. Exiting.]");
		close(sockfd);
		return(-1);
	}
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[StartServer]-[Port %s bound]", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.PORT"));
	
	listen(sockfd, 250);
	len = sizeof(cli_addr);
	for(;;)
	{
		clientsockfd = accept(sockfd,(struct sockaddr*)&cli_addr,&len);
		if(clientsockfd > 0)
		{
			tCIP = ConvertL2IP(cli_addr.sin_addr.s_addr);
			if(CheckIPAccess(cli_addr.sin_addr.s_addr))
			{
				MyDebug(__LINE__, __FILE__, LOG_SOCKET,  "[SocketServer]-[Client connected]-[%s]", tCIP);
				do
				{
					FreeChildrenCount = FreeChild(CHILDREN, BIRTH);
					if(FreeChildrenCount != -1)
					{
						child = fork();
						if(child == 0)
						{
							MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SocketServer]-[Client Forked]");
							
							SendBuff = dsprintf(SendBuff, "MIDASd v%s connection accepted\n", MIDASVER);
							SendToSocket(clientsockfd, SendBuff, MIDAS_EOL, 0);
							SendBuff = dfree(SendBuff);
							
							ClientString = RecvFromSocket(clientsockfd, MIDAS_EOL, 1);
							if(ClientString)
							{
								MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SocketServer]-[Client Sent %s]", ClientString);
								
								ClientXMLRequest = StringToXMLTree(ClientString);
								ClientString = dfree(ClientString);

								if(ClientXMLRequest)
								{
									ClientResponse = ProcessClientRequest(ClientXMLRequest);
									if(ClientResponse)
									{
										SendToSocket(clientsockfd, ClientResponse, MIDAS_EOL, 1);
									}
									ClientResponse = dfree(ClientResponse);
								} else {
									MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SocketServer]-[Invalid Client Request]");
									SendToSocket(clientsockfd, "<SERVER><TYPE>INVALID</TYPE></SERVER>", MIDAS_EOL, 1);
								}
								ClientXMLRequest = DelXMLLeafTree(ClientXMLRequest);
								
								// Wait for the client <CLOSE>
								ClientString = RecvFromSocket(clientsockfd, MIDAS_EOL, 1);
								ClientString = dfree(ClientString);
							}

							MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SocketServer]-[Client Closed]");
							close(clientsockfd);
							exit(0);
						} else {
							close(clientsockfd);
							AddChild(child, CHILDREN, BIRTH);
						}
					} else {
						usleep(100);
						continue;
					}
				}while(FreeChildrenCount == -1);
			} else {
				MyDebug(__LINE__, __FILE__, LOG_SOCKET,  "[SocketServer]-[IP Access denied to : %s]", tCIP);
				close(clientsockfd);
			}
			tCIP = dfree(tCIP);
		}
	}
	return(0);
}

/******************************************************************************
*       int CheckIPAccess(unsigned clientip)
*		Checks the connecting IP address against the defined IP blocks 
*		to verify that its allowed to connect. Returns 1 if connection
*		allowed or 0 if not allowed.
*
*	Args :
*		unsigned clientip	The connecting (numeric) IP address.
*
*       Updated :
******************************************************************************/						        
int CheckIPAccess(unsigned clientip)
{
	int  status = 0;
	struct XMLLeaf * LocalXMLLeaf;
	unsigned tIP;
	unsigned tMASK;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[CheckIPAccess]-[Enter]");

	LocalXMLLeaf = ConfigXMLTree;
	
	while(LocalXMLLeaf)
	{
		LocalXMLLeaf = FindXMLLeafByTag(LocalXMLLeaf, "CONFIG.AccessIP.IP");
		if(LocalXMLLeaf)
		{
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[CheckIPAccess]-[IP:%s]", LocalXMLLeaf->Value);

			tIP = ConvertIP2L(LocalXMLLeaf->Value);

			LocalXMLLeaf = FindXMLLeafByTag(LocalXMLLeaf, "CONFIG.AccessIP.IP.MASK");
			if(LocalXMLLeaf)
			{
				tMASK = (MyPow(2, Myatoi(LocalXMLLeaf->Value)) -1);
				if((clientip & tMASK) == tIP)
				{
					status = 1;
					break;
				}
			}
		}
	}
	return(status);	
}
