/******************************************************************************
* Copyright (c) 2003 Tom Cashmore
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to
* deal in the Software without restriction, including without limitation the
* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
* sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.

* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*
*       Applisation : MIDASn
*       Filename : MIDASn.c
* 		Funstions for main portion of MIDASn application
*
*       Updated :
*               11/10/2003 - Added function and header comments.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <signal.h>
#include <netdb.h>
#include <sys/time.h>
#include <linux/if.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <linux/icmp.h>
#include <linux/if_ether.h>
#include <time.h>


int DEBUGLEVEL 	= 0;
int LOG_INFO 	= 1;
int LOG_SOCKET 	= 2;
int LOG_XML	= 4;
int LOG_CHECK   = 8;
int LOG_FORK    = 16;

struct XMLLeaf * ConfigXMLTree = 0;
char APPNAME[] = "MIDASn";
char CFGPATH[] = "/usr/local/MIDAS/etc/MIDASn.cf";
char * KEYFILE = 0;
int SENDDELAY = 60;
extern char MIDAS_EOL[];

struct etherpacket {
	struct ethhdr		eth;	
	struct iphdr		ip;
	struct tcphdr		tcp;
	char			data[8192];
};

struct PortStat {
	char		* Protocol;
	unsigned	SrcPort;
	unsigned	DstPort;
	unsigned	Count;
	unsigned	Bytes;
	struct	PortStat * NextStat;
};
	
struct NetStat {
	unsigned	TotalPkt;
	unsigned	TotalPktCount;
	unsigned	TCPPkt;
	unsigned	TCPPktCount;
	unsigned	UDPPkt;
	unsigned	UDPPktCount;
	unsigned	ICMPPkt;
	unsigned	ICMPPktCount;
	unsigned	OtherPkt;
	unsigned	OtherPktCount;
	struct	PortStat * PortStats;
};
	
#include "../Common/Commoninclude.h"
#include "includes.h"

/******************************************************************************
*       int main(int argc, char *argv[])
*		The main() function for MIDASn
*
*	Args :
*		int argc	Argument count.
*		char * argv[]	Argument value array.
*
*       Updated : 
******************************************************************************/
int main(int argc, char *argv[])
{
	char * Query = 0;
	char * Response = 0;
	struct XMLLeaf * PortLeaf = 0;
	
	ConfigXMLTree = ReadConfig(CFGPATH);
	
	if(!ConfigXMLTree)
	{
		printf("! Unable to open %s\n", CFGPATH);
		exit(0);
	}
	
	DEBUGLEVEL = Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DEBUGLEVEL"));
	KEYFILE = dstrcpy(KEYFILE, ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.KEYFILE"));
	
	if(!ParseCommandLine(argc, argv))
	{
		exit(0);
	}
	
	if(Myatoi(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.DAEMON")))
	{
		if(fork())
		{
			exit(0);
		}
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[main]-[MIDASn %s starting]", MIDASVER);

	Query = dsprintf(Query, "<CLIENT><HID>%s</HID><TYPE>RETURNNETMON</TYPE></CLIENT>", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"));
	Response = SendToServer(Query);
	Query = dfree(Query);

	if(Response)
	{
		PortLeaf = StringToXMLTree(Response);
		DebugXMLTree("PortLeaf", PortLeaf, 1);
		Sniffer(PortLeaf);
		DelXMLLeafTree(PortLeaf);
	}
	Response = dfree(Response);
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[main]-[MIDASn %s exiting]", MIDASVER);
	
	DelXMLLeafTree(ConfigXMLTree);
	
	exit(0);
}

/******************************************************************************
*       int initdevice(char * device)
*		Builds socket and puts specified ethernet device into 
*		promiscuous mode.
*
*	Args :
*		char * device		Device to modify
*
*       Updated :
******************************************************************************/
int initdevice(char * device)
{
	int if_fd =0;
	struct ifreq ifr;

	if_fd = socket(AF_INET,SOCK_PACKET,  htons(0x0800));

	strcpy(ifr.ifr_name, device);
	ioctl(if_fd, SIOCGIFFLAGS, &ifr);

	ifr.ifr_flags |= IFF_PROMISC;
	ioctl(if_fd, SIOCSIFFLAGS, &ifr);

	return if_fd;
}

/******************************************************************************
*	void sniffer(struct XMLLeaf * PortLeaf)       
*		Packet sniffer function. This grabs the packets and builds the 
*		stats based on the PortLeaf passed to it.
*
*	Args :
*		struct XMLLeaf * PortLeaf	XML linked list of the ports
*						to monitor.
*
*       Updated :
******************************************************************************/
void Sniffer(struct XMLLeaf * PortLeaf)
{
	int if_eth_fd = 0; 
	struct etherpacket ep;
	struct sockaddr dest;
	struct timeval timeout;
	fd_set rd;
	int dlen;
	struct NetStat * NetworkStat = 0;
	unsigned long LastStatSend = 0;
	int sock_status = 0;
			
	NetworkStat = (struct NetStat *) malloc(sizeof(struct NetStat));
	bzero(NetworkStat, sizeof(struct NetStat));
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[Sniffer]-[Enter]-[Dev:%s]", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.ETHDEV"));
	
	if_eth_fd = initdevice(ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.ETHDEV"));
	
	if(if_eth_fd)
	{
		for(;;)
		{
			bzero(&dest,sizeof(dest));
		
			dlen = 0;
			
			timeout.tv_sec=0;
			timeout.tv_usec=0;
			
			while(timeout.tv_sec==0 && timeout.tv_usec==0)
			{
				FD_ZERO(&rd);
				FD_SET(if_eth_fd,&rd);

				timeout.tv_sec=10;
				timeout.tv_usec=0;
				
				sock_status = select(if_eth_fd+1, &rd, NULL, NULL, &timeout);
		
				if(sock_status)
				{
					recvfrom(if_eth_fd, &ep,sizeof(ep), 0, &dest, &dlen);
				}
			}	
	
			if(ep.eth.h_proto == ntohs(ETH_P_IP))
			{
				NetworkStat = UpdateStats(&ep, NetworkStat);
				NetworkStat = UpdateStats_Ports(&ep, NetworkStat, PortLeaf);
					
				if(time(NULL) >= LastStatSend + SENDDELAY)
				{
					SendStats(NetworkStat);
					
					NetworkStat->PortStats = UpdateStats_PurgeStats(NetworkStat->PortStats);
					bzero(NetworkStat, sizeof(struct NetStat));
					LastStatSend = time(NULL);
				}
			}
		}
	} else {
		MyDebug(__LINE__, __FILE__, LOG_INFO, "[Sniffer]-[Unable to open %s]", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.ETHDEV"));
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[Sniffer]-[Exit]");
	
}

/******************************************************************************
*       struct NetStat * UpdateStats(struct etherpacket * ep, struct NetStat * NetworkStat)
*		Determinds the type of packet we are looking at then increments the 
*		network stats.
*
*	Args :
*		struct etherpacket * ep		Ethernet packet.
*		struct NetStat * NetworkStat	Network stat struct.
*
*       Updated :
******************************************************************************/
struct NetStat * UpdateStats(struct etherpacket * ep, struct NetStat * NetworkStat)
{
	struct iphdr * ip;

	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats]-[Enter]");
	
	ip = (struct iphdr *) (((unsigned long) &ep->ip) - 2);
	
	NetworkStat->TotalPkt += htons(ip->tot_len);
	NetworkStat->TotalPktCount++;
		
	switch(ip->protocol)
	{
		case 1:
			NetworkStat->ICMPPkt += htons(ip->tot_len);
			NetworkStat->ICMPPktCount++;
			break;
		case 6:
			NetworkStat->TCPPkt += htons(ip->tot_len);
			NetworkStat->TCPPktCount++;
			break;
		case 17:
			NetworkStat->UDPPkt += htons(ip->tot_len);
			NetworkStat->UDPPktCount++;
			break;
		default:
			NetworkStat->OtherPkt += htons(ip->tot_len);
			NetworkStat->OtherPktCount++;
			break;
	}
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats]-[Exit]");
	
	return(NetworkStat);
}

/******************************************************************************
*       struct NetStat * UpdateStats_Ports(struct etherpacket * ep, struct NetStat * NetworkStat, struct XMLLeaf * PortLeaf)
*		Updates the port statistics. Uses the PortLeaf to figure out if
*		its a port were wathcing for. If so then it updates the stats.
*
*	Args :
*		struct etherpacket * ep		Ethernet packet.
*		struct NetStat * NetworkStat	Network stats struct.
*		struct XMLLeaf * PortLeaf	XML LL contatincing the ports to 
*						watch.
*
*       Updated :
******************************************************************************/
struct NetStat * UpdateStats_Ports(struct etherpacket * ep, struct NetStat * NetworkStat, struct XMLLeaf * PortLeaf)
{
	struct iphdr * ip;
	struct tcphdr * tcp;
	struct udphdr * udp;
	struct icmphdr * icmp;
	unsigned SrcPort = 0;
	unsigned DstPort = 0;
	unsigned Len = 0;
	int Status = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_Ports]-[Enter]");
	
	if(ep)
	{		
		ip = (struct iphdr *) (((unsigned long) &ep->ip) - 2);
	
		switch(ip->protocol)
		{
			case 1:
				icmp = (struct icmphdr *) (((unsigned long) &ep->tcp) - 2);
				break;
			case 6:
				tcp = (struct tcphdr *) (((unsigned long) &ep->tcp) - 2);
			
				SrcPort = htons(tcp->source);
				DstPort = htons(tcp->dest);
				Len = htons(ip->tot_len);
			
				if((Status = UpdateStats_WatchPort("TCP", SrcPort, DstPort, PortLeaf)))
				{
					switch(Status)
					{
						case 1:
							SrcPort = 0;
							break;
						case 2: 
							DstPort = 0;
							break;
					};
							
					NetworkStat->PortStats = UpdateStats_IncrementPort(NetworkStat->PortStats, "TCP", SrcPort, DstPort, Len);
				}
				break;
			case 17:
				udp = (struct udphdr *) (((unsigned long) &ep->tcp) - 2);
				
				SrcPort = htons(udp->source);
				DstPort = htons(udp->dest);
				Len = htons(ip->tot_len);
				
				if((Status = UpdateStats_WatchPort("UDP", SrcPort, DstPort, PortLeaf)))
				{
					switch(Status)
					{
						case 1:
							SrcPort = 0;
							break;
						case 2: 
							DstPort = 0;
							break;
					};
					
					NetworkStat->PortStats = UpdateStats_IncrementPort(NetworkStat->PortStats, "UDP", SrcPort, DstPort, Len);
				}
				break;
			default:
				break;
		}
	}
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_Ports]-[Exit]");
	
	return(NetworkStat);
}

/******************************************************************************
*       int UpdateStats_WatchPort(char * Protocol, unsigned SrcPort, unsigned DstPort, struct XMLLeaf * PortLeaf)
*		Returns 1 if we should be wathing this port or 0 if not.
*
*	Args :
*		char * Protocol			Protocol (TCP, UDP, ICMP)
*		unsigned SrcPort		Source port of packet.
*		unsigned DstPort		Destination port of packet.
*		struct XMLLeaf * PortLeaf	XML LL containing watched port.
*
*       Updated :
******************************************************************************/
int UpdateStats_WatchPort(char * Protocol, unsigned SrcPort, unsigned DstPort, struct XMLLeaf * PortLeaf)
{
	struct XMLLeaf * LocalLeaf = 0;
	struct XMLLeaf * SubLeaf = 0;
	char * tLeafTag = 0;
	int Status = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_WatchPort]-[Enter]-[Proto:%s]-[SrcPort:%u]-[DstPort:%u]", Protocol, SrcPort, DstPort);
	
	LocalLeaf = PortLeaf;
	if(LocalLeaf)
	{
		while(LocalLeaf)
		{
			SubLeaf = BuildSubXMLTree(LocalLeaf, ReturnXMLLeafID(LocalLeaf, "SERVER.NETMON.PORTS"), 1);
			if(SubLeaf)
			{
				if(SrcPort)
				{
					tLeafTag = dsprintf(tLeafTag, "SERVER.NETMON.PORTS.%s.SRCPORT", Protocol);
					if(SrcPort == Myatoll(ReturnXMLLeafValue(SubLeaf, tLeafTag)))
					{
						Status = 2;
					}
					tLeafTag = dfree(tLeafTag);
				}
				
				if(DstPort)
				{
					tLeafTag = dsprintf(tLeafTag, "SERVER.NETMON.PORTS.%s.DSTPORT", Protocol);
					if(DstPort == Myatoll(ReturnXMLLeafValue(SubLeaf, tLeafTag)))
					{
						Status = 1;
					}
					tLeafTag = dfree(tLeafTag);
				}
				
				LocalLeaf = FindXMLLeafByID(LocalLeaf, SubLeaf->ID);
				SubLeaf = DelSubXMLTree(SubLeaf);
				
				if(Status)
				{
					break;
				}
			}
			
			if(LocalLeaf)
			{
				LocalLeaf = LocalLeaf->NextLeaf;
			}
		}
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_WatchPorts]-[Status:%d]-[Exit]", Status);
	
	return(Status);	
}

/******************************************************************************
*       struct PortStat * UpdateStats_IncrementPort(struct PortStat * PortStats, char * Protocol, unsigned SrcPort, unsigned DstPort, unsigned Len)
*		Increment port stats with packet information.
*
*	Args :
*		struct PortStat * PortStats	Stored port statistics
*		char * Protocol			Packet porotocol.
*		unsigned SrcPort		Source port of packet.
*		unsigned DstPort		Destination port of packet.
*		unsigned Len			Length of packet.
*
*       Updated :
******************************************************************************/
struct PortStat * UpdateStats_IncrementPort(struct PortStat * PortStats, char * Protocol, unsigned SrcPort, unsigned DstPort, unsigned Len)
{
	struct PortStat * LocalStats = 0;
	struct PortStat * NewStats = 0;
	struct PortStat * PrevStats = 0;
	int Found = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_IncrementPort]-[Proto:%s]-[SrcPort:%u]-[DstPort:%u]-[Len:%u]", Protocol, SrcPort, DstPort, Len);
	
	LocalStats = PortStats;
	
	if(LocalStats)
	{
		while(LocalStats)
		{
			if(!strcasecmp(LocalStats->Protocol, Protocol) &&
			    LocalStats->SrcPort == SrcPort &&
			    LocalStats->DstPort == DstPort)
			{
				MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpDateStats_IncrementPort]-[Incrementing Existing Element]");
				
				Found = 1;
				LocalStats->Count++;
				LocalStats->Bytes=+Len;
				break;
			}
			LocalStats = LocalStats->NextStat;
		}
	
		if(!Found)
		{
			MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_IncrementPort]-[Adding New Element]");
			
			LocalStats = PortStats;
			if(LocalStats)
			{
				while(LocalStats)
				{
					PrevStats = LocalStats;
					LocalStats = LocalStats->NextStat;
				}
			}

			NewStats = (struct PortStat *) drealloc((char *)NewStats, sizeof(struct PortStat));
		
			NewStats->Protocol = dstrcpy(NewStats->Protocol, Protocol);
			NewStats->DstPort = DstPort;
			NewStats->SrcPort = SrcPort;
			NewStats->Bytes = Len;
			NewStats->Count = 1;
		
			if(PrevStats)
			{
				PrevStats->NextStat = NewStats;
			}
		}
	} else {
		MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_IncrementPort]-[First Element]");
		
		NewStats = (struct PortStat *) drealloc((char *)NewStats, sizeof(struct PortStat));
		
		NewStats->Protocol = dstrcpy(NewStats->Protocol, Protocol);
		NewStats->DstPort = DstPort;
		NewStats->SrcPort = SrcPort;
		NewStats->Bytes = Len;
		NewStats->Count = 1;
		
		PortStats = NewStats;
	}	

	return(PortStats);
}

/******************************************************************************
*       struct PortStat * UpdateStats_PurgeStats(struct PortStat * PortStats)
*		Clears/Frees Port statistics.
*
*	Args :
*		struct PortStat * PortStats	The Port Stats to clear/free.
*
*       Updated :
******************************************************************************/
struct PortStat * UpdateStats_PurgeStats(struct PortStat * PortStats)
{
	struct PortStat * LocalStats = 0;
	struct PortStat * NextStats = 0;
	unsigned Count = 0;
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_PurgeStats]-[Enter]");
	
	LocalStats = PortStats;
	
	if(LocalStats)
	{
		while(LocalStats->NextStat)
		{
			Count++;
			NextStats = LocalStats->NextStat;
			
			LocalStats->Protocol = dfree((char *) LocalStats->Protocol);
			LocalStats = (struct PortStat *) dfree((char *) LocalStats);
			
			LocalStats = NextStats;
		}
		
		if(LocalStats)
		{
			LocalStats = (struct PortStat *) dfree((char*) LocalStats);
		}
	}
	
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[UpdateStats_PurgeStats]-[Exit]-[Elements:%u]", Count);
	return(NULL);
}

/******************************************************************************
*       void SendStats(struct NetStat * NetworkStat)
*		Builds NETSTAT query and sends to MIDASd.
*
*	Args :
*		struct NetStat * NetworkStat	The Network stats to use to 
*						build query with.
*
*       Updated :
******************************************************************************/			
void SendStats(struct NetStat * NetworkStat)
{
	char * Query = 0;
	char * tQuery = 0;
	char * Response = 0;
	struct PortStat * PortStats = 0;
	unsigned Port = 0;
	unsigned PortCount = 0;
			
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[SendStats]-[Enter]");

	Query = dsprintf(Query, "<CLIENT><HID>%s</HID><TYPE>NETSTAT</TYPE><NETSTAT>", ReturnXMLLeafValue(ConfigXMLTree, "CONFIG.HID"));
	
	if(NetworkStat)
	{
		// Total
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>BYTES</ID><ORDER>0</ORDER><LABEL>Total Bytes</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->TotalPkt);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
		
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>COUNT</ID><ORDER>0</ORDER><LABEL>Total Packet Count</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->TotalPktCount);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
		
		// TCP
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>BYTES</ID><ORDER>1</ORDER><LABEL>TCP Bytes</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->TCPPkt);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
		
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>COUNT</ID><ORDER>1</ORDER><LABEL>TCP Packet Count</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->TCPPktCount);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
				
		// UDP
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>BYTES</ID><ORDER>2</ORDER><LABEL>UDP Bytes</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->UDPPkt);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
		
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>COUNT</ID><ORDER>2</ORDER><LABEL>UDP Packet Count</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->UDPPktCount);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
				
		// ICMP
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>BYTES</ID><ORDER>3</ORDER><LABEL>ICMP Bytes</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->ICMPPkt);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
		
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>COUNT</ID><ORDER>3</ORDER><LABEL>ICMP Packet Count</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->ICMPPktCount);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
		
		// Other
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>BYTES</ID><ORDER>4</ORDER><LABEL>Other Bytes</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->OtherPkt);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
		
		tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>COUNT</ID><ORDER>4</ORDER><LABEL>Other Packet Count</LABEL><DATAVAL>%u</DATAVAL></STAT>",
					  NetworkStat->OtherPktCount);
		Query = dstrcat(Query, tQuery);
		tQuery = dfree(tQuery);
		
		if(NetworkStat->PortStats)
		{
			PortStats = NetworkStat->PortStats;
			
			while(PortStats)
			{
				if(PortStats->SrcPort)
				{
					Port = PortStats->SrcPort;
				} else {
					Port = PortStats->DstPort;
				}
								
				tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>PORT_COUNT_%s_%u</ID><ORDER>%u</ORDER><LABEL>%s Port %u (Packets)</LABEL><DATAVAL>%u</DATAVAL></STAT>",
						          PortStats->Protocol,
						          Port,
						          PortCount,
						          PortStats->Protocol,
						          Port,
						          PortStats->Count);
				Query = dstrcat(Query, tQuery);
				tQuery = dfree(tQuery);
				
				tQuery = dsprintf(tQuery, "<STAT><TYPE>NETMON</TYPE><ID>PORT_BYTES_%s_%u</ID><ORDER>%u</ORDER><LABEL>%s Port %u (Bytes)</LABEL><DATAVAL>%u</DATAVAL></STAT>",
						          PortStats->Protocol,
						          Port,
						          PortCount, 
						          PortStats->Protocol,
						          Port,
						          PortStats->Bytes);
				Query = dstrcat(Query, tQuery);
				tQuery = dfree(tQuery);

				PortStats = PortStats->NextStat;
				
				PortCount++;
				
			}
		}
	}
	
	Query = dstrcat(Query, "</NETSTAT></CLIENT>");
	
	Response = SendToServer(Query);
	Response = dfree(Response);
	
	Query = dfree(Query);
				
	MyDebug(__LINE__, __FILE__, LOG_INFO, "[SendStats]-[Exit]");	
}

/******************************************************************************
*       struct XMLLeaf * ReadConfig(char * filename)
*		Reads XML config file and returns an XML linked list of 
*		contents.
*
*	Args :
*		char * filename		Path and filename of XML config file.
*
*       Updated :
******************************************************************************/	
struct XMLLeaf * ReadConfig(char * filename)
{
	FILE *fp = 0;
	struct XMLLeaf * LocalLeaf = 0;
	char * FileString = 0;
		
	fp = fopen(filename, "r");
	if(fp)
	{
		FileString = dfiletostring(fp);
		if(FileString)
		{
			LocalLeaf = StringToXMLTree(FileString);
		}
		FileString = dfree(FileString);
		fclose(fp);
	}
	return(LocalLeaf);	
}		

/******************************************************************************
*       char * SendToServer(char * SendBuff)
*		Sends a string buffer to MIDASd. Uses MIDAS_EOL as end of line.
*
*	Args :
*		char * SendBuff		The Sting buffer to send.
*
*       Updated :
******************************************************************************/
char * SendToServer(char * SendBuff)
{
	struct XMLLeaf * LocalLeaf;
	int sock, connected;
	char * IP = 0;
	int Port = 0;
	char * RecvBuff = 0;
	struct sockaddr_in addr;
	struct hostent * hostinfo = 0;
		
	if(SendBuff)
	{
		LocalLeaf = ConfigXMLTree;
		
		while(LocalLeaf)
		{
			LocalLeaf = FindXMLLeafByTag(LocalLeaf, "CONFIG.SERVERIP.IP");
			if(LocalLeaf)
			{
				IP = LocalLeaf->Value;
				LocalLeaf = FindXMLLeafByTag(LocalLeaf, "CONFIG.SERVERIP.IP.PORT");
				if(LocalLeaf)
				{
					Port = Myatoi(LocalLeaf->Value);
					
					MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SendToServer]-[Connecting to : %s:%d]", IP, Port);
					
					sock = socket(AF_INET, SOCK_STREAM, 0);
        				if(sock != -1)
        				{
        					hostinfo = gethostbyname(IP);
        					
						addr.sin_family = AF_INET;
						addr.sin_port = htons(Port);
						addr.sin_addr = *(struct in_addr *) *hostinfo->h_addr_list;;
						bzero(&(addr.sin_zero), 8);
					
						connected = connect(sock, (struct sockaddr *) &addr, sizeof(struct sockaddr));

						if(connected != -1)
						{
							// [TCC] Grab the welcome line
							RecvBuff = RecvFromSocket(sock, MIDAS_EOL, 0);
							RecvBuff = dfree(RecvBuff);
								
							SendToSocket(sock, SendBuff, MIDAS_EOL, 1);
							
							RecvBuff = RecvFromSocket(sock, MIDAS_EOL, 1);
							MyDebug(__LINE__, __FILE__, LOG_SOCKET, "[SendtoServer]-[RecvLen:%d]", dstrlen(RecvBuff));
							
							SendToSocket(sock, "<CLIENT><CLOSE></CLOSE></CLIENT>", MIDAS_EOL, 1);
							sleep(1);
						}
						close(sock);
					} else {
						LocalLeaf = FindXMLLeafByTag(LocalLeaf, "CONFIG.SERVERIP.IP.FAILOVER");
						if(LocalLeaf)
						{
							if(!Myatoi(LocalLeaf->Value))
							{
								break;
							}
						}
					}
				}	
			}
		}
	}
	return(RecvBuff);
}

/******************************************************************************
*       int ParseCommandLine(int argc, char *argv[])
*		Parses the command line arguments.
*
*	Args :
*		int argc	Argument count.
*		char *argv[]	Argument string array.
*
*       Updated :
******************************************************************************/
int ParseCommandLine(int argc, char *argv[])
{
	int opt = 0;
	int Status = 1;
		
	while(opt != -1)
	{
		opt = getopt(argc, argv, "VhD:T");
		switch(opt)
		{
			case 'V':
				printf("%s %s\n", APPNAME, MIDASVER);
				Status = 0;
				break;
			case 'h':
				printf("%s %s\n", APPNAME, MIDASVER);
				printf("\t-D#\tSet the Debug level\n");
				printf("\t-h\tShow (this) help screen\n");
				printf("\t-T\tShow configuration\n");
				printf("\t-V\tShow version\n");
				Status = 0;
				break;
			case 'D':
				DEBUGLEVEL = Myatoi(optarg);
				break;
			case 'T':
				ShowConfig();
				Status = 0;
				break;
		}
	}
	return(Status);
}

/******************************************************************************
*       void ShowConfig(void)
*		Shows current configurations. This is triggered with the -T
*		command line arg.
*
*
*       Updated :
******************************************************************************/
void ShowConfig(void)
{
        printf("%s %s\n", APPNAME, MIDASVER);
        printf("------------------------------------------------------------------\n");
        printf("[Contents of %s.cf file]\n", APPNAME);
        DebugXMLTree("MIDASn.cf", ConfigXMLTree, 0);
}
