#
#	New AlertOnRed field
#	Set all to 'Off' by default
#
ALTER TABLE MIDAS_Checks ADD AlertOnRed int;
UPDATE MIDAS_Checks SET AlertOnRed=0;

#
#	Increase the HostName and IP Address fields.
#
ALTER TABLE MIDAS_Hosts MODIFY HostName char(255);
ALTER TABLE MIDAS_Hosts MODIFY IPAddr char(255);

#
#	Add all the RRD related Tables
#
DROP TABLE IF EXISTS RRD_MIDAS_ID;
CREATE TABLE RRD_MIDAS_ID (
	DSID	int NOT NULL,
	Type	char(50) NOT NULL,
	ID1	char(50) NOT NULL,
	ID2	char(50) NOT NULL,
	ID3	char(50) NOT NULL,
	Primary Key(DSID, Type, ID1, ID2, ID3)
);

DROP TABLE IF EXISTS RRD_DataSource;
CREATE TABLE RRD_DataSource (
	DSID			int NOT NULL,
	Sequences		int,
	Intervals		int,
	Title			char(255),
	Primary Key(DSID)
);

DROP TABLE IF EXISTS RRD_DataValues;
CREATE TABLE RRD_DataValues (
	DSID		int NOT NULL,
	Sequence	int NOT NULL,
	TimeStamp	int,
	Value		float,
	AverageValue	float,
	DerivedValue	float,
	Primary Key(DSID, Sequence)
);

DROP TABLE IF EXISTS RRD_ArchiveSource;
CREATE TABLE RRD_ArchiveSource (
	DAID 		int NOT NULL AUTO_INCREMENT,
	DSID		int,
	Sequences	int,
	Intervals	int,
	Primary Key(DAID)
);

DROP TABLE IF EXISTS RRD_ArchiveValues;
CREATE TABLE RRD_ArchiveValues (
	DAID		int NOT NULL,
	Sequence	int NOT NULL,
	TimeStamp	int,
	Value		float,
	AverageValue	float,
	DerivedValue	float,
	Primary Key(DAID, Sequence)
);

DROP TABLE IF EXISTS MIDAS_Reg_Modules;
#
CREATE TABLE MIDAS_Reg_Modules (
  module_name 		varchar(100) NOT NULL default '',
  module_loc 		varchar(100) NOT NULL default '',
  module_req 		varchar(255) NOT NULL default '',
  module_priority 	int(2) NOT NULL default '9',
  module_init 		varchar(100) NOT NULL default '',
  PRIMARY KEY (module_name)
);


DROP TABLE IF EXISTS MIDAS_User_Activity;
#
CREATE TABLE MIDAS_User_Activity (
  uid 		int(11) NOT NULL default '0',
  accessLevel 	char(3) default NULL,
  remoteaddr 	varchar(15) default NULL,
  sesskey 	varchar(255) NOT NULL default '',
  stamp 	int(20) NOT NULL default '0',
  PRIMARY KEY (uid),
  KEY sesskey (sesskey)
);

# UPGRADE MIDAS_Users
#
ALTER TABLE MIDAS_Users CHANGE UID uid INT( 11 ) NOT NULL AUTO_INCREMENT;
ALTER TABLE MIDAS_Users CHANGE Login username VARCHAR( 40 ) NOT NULL;
ALTER TABLE MIDAS_Users CHANGE Password passwd VARCHAR( 40 ) DEFAULT NULL;
ALTER TABLE MIDAS_Users CHANGE Email email VARCHAR( 40 ) DEFAULT NULL;
ALTER TABLE MIDAS_Users DROP Admin;
ALTER TABLE MIDAS_Users ADD accessLevel INT( 11 ) NOT NULL DEFAULT '0' AFTER email;
ALTER TABLE MIDAS_Users CHANGE LastLogin lastAccess INT( 20 ) DEFAULT NULL;
ALTER TABLE MIDAS_Users DROP LastIP;
ALTER TABLE MIDAS_Users DROP MiniMe;
ALTER TABLE MIDAS_Users CHANGE Enabled enabled INT( 11 ) DEFAULT NULL;
ALTER TABLE MIDAS_Users ADD UNIQUE (username);

#
# Giving every user Super_User Rights (Needed so the user who's logging in 
# to finish the upgrade can function properly). 
UPDATE MIDAS_Users SET accessLevel=255;

#
# Updating all current users to blank passwords
UPDATE MIDAS_Users SET passwd=MD5('password');

#
# Clearing all lastAccess information (Since we converted the field type from character based to int
UPDATE MIDAS_Users SET lastAccess='';

#
# Adding Indexes to MIDAS_IDS_Events and MIDAS_Check_Events
ALTER TABLE `MIDAS_IDS_Events` ADD INDEX(`TimeStamp`);

ALTER TABLE `MIDAS_Check_Events` ADD INDEX(`HID`);
ALTER TABLE `MIDAS_Check_Events` ADD INDEX(`CID`);
ALTER TABLE `MIDAS_Check_Events` ADD INDEX(`TID`);
ALTER TABLE `MIDAS_Check_Events` ADD INDEX(`TimeStamp`);
